

from __future__ import print_function



# ImageD11_v1.0 Software for beamline ID11
# Copyright (C) 2005-2007  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0211-1307  USA



"""
Script for peaksearching images from the command line

Uses the connectedpixels extension for finding blobs above a threshold
and the blobcorrector(+splines) for correcting them for spatial distortion

Defines one function (peaksearch) which might be reused
"""

# For benchmarking
import time
reallystart = time.time()

from six.moves import queue
# import threading
from math import sqrt
import sys , glob , os.path
import numpy

# Generic file format opener from fabio
import fabio
from fabio.openimage import openimage
from fabio import fabioimage

from ImageD11 import blobcorrector, ImageD11options
from ImageD11.correct import correct
from ImageD11.labelimage import labelimage
from ImageD11 import ImageD11_thread 
ImageD11_thread.stop_now = False


class timer:
    def __init__(self):
        self.start = time.time()
        self.now = self.start
        self.msgs = []
    def msg(self,msg):
        self.msgs.append(msg)
    def tick(self,msg=""):
        now = time.time()
        self.msgs.append("%s %.2f/s"%(msg,now-self.now))
        self.now = now
    def tock(self,msg=""):
        self.tick(msg)
        print(" ".join(self.msgs),"%.2f/s"% (self.now-self.start))
        sys.stdout.flush()


def peaksearch( filename , 
                data_object , 
                corrector , 
                thresholds ,  
                labims ):
    """
    filename  : The name of the image file for progress info 	
    data_object : Fabio object containing data and header 
    corrector : spatial and dark/flood , linearity etc
    
    thresholds : [ float[1], float[2] etc]

    labims : label image objects, one for each threshold
    
    """
    t = timer()
    picture = data_object.data.astype(numpy.float32)
    
    assert "Omega" in data_object.header, "Bug in peaksearch headers"

    for lio in list(labims.values()):
        f = lio.sptfile
        f.write("\n\n# File %s\n" % (filename))
        f.write("# Frame %d\n" % (data_object.currentframe) )
        f.write("# Processed on %s\n" % (time.asctime()))
        try:
            f.write("# Spatial correction from %s\n" % (corrector.splinefile))
            f.write("# SPLINE X-PIXEL-SIZE %s\n" % (str(corrector.xsize)))
            f.write("# SPLINE Y-PIXEL-SIZE %s\n" % (str(corrector.ysize)))
        except:
            pass
        for item in list(data_object.header.keys()):
            if item == "headerstring": # skip
                continue
            try:
                f.write("# %s = %s\n" % (item,
                        str(data_object.header[item]).replace("\n"," ")))
            except KeyError:
                pass

    # Get the rotation angle for this image
    ome = float(data_object.header["Omega"])
    # print "Reading from header"
    #
    # Now peaksearch at each threshold level    
    t.tick(filename)
    for threshold in thresholds:
        labelim = labims[threshold]
        f = labelim.sptfile
        if labelim.shape != picture.shape:
            raise "Incompatible blobimage buffer for file %s" %(filename)
        #
        #
        # Do the peaksearch
        f.write("# Omega = %f\n"%(ome))
        labelim.peaksearch(picture, threshold, ome)
        f.write("# Threshold = %f\n"%(threshold))
        f.write("# npks = %d\n"%(labelim.npk))
        #
        if labelim.npk > 0:
            labelim.output2dpeaks(f)
        labelim.mergelast()
        t.msg("T=%-5d n=%-5d;" % (int(threshold),labelim.npk)) 
        # Close the output file
    # Finish progress indicator for this file
    t.tock()
    sys.stdout.flush()
    return None 


def peaksearch_driver(options, args):
    """
    To be called with options from command line
    """
    ################## debugging still
    for a in args:
        print("arg: "+str(a)+","+str(type(a)))
    for o in list(options.__dict__.keys()): # FIXME
        if getattr(options,o) in [ "False", "FALSE", "false" ]:
            setattr(options,o,False)
        if getattr(options,o) in [ "True", "TRUE", "true" ]:
            setattr(options,o,True)
        print("option:",str(o),str(getattr(options,o)),",",\
        str(type( getattr(options,o) ) ))
    ###################
    print("This peaksearcher is from",__file__)

    if options.killfile is not None and os.path.exists(options.killfile):
        print("The purpose of the killfile option is to create that file")
        print("only when you want peaksearcher to stop")
        print("If the file already exists when you run peaksearcher it is")
        print("never going to get started")
        raise ValueError("Your killfile "+options.killfile+" already exists")

    if options.thresholds is None:
        raise ValueError("No thresholds supplied [-t 1234]")

    if len(args) == 0  and options.stem is None:
        raise ValueError("No files to process")

        # What to do about spatial
 
    if options.perfect=="N" and os.path.exists(options.spline):
        print("Using spatial from",options.spline)
        corrfunc = blobcorrector.correctorclass(options.spline)
    else:
        print("Avoiding spatial correction")
        corrfunc = blobcorrector.perfect()



    # Get list of filenames to process
    # if len(args) > 0 :
    #     # We no longer assume unlabelled arguments are filenames 
    #     file_series_object = file_series.file_series(args)

    # This is always the case now
    corrfunc.orientation = "edf"
    
    if options.format in ['bruker', 'BRUKER', 'Bruker']:
        extn = ""
        if options.perfect is not "N":
            print("WARNING: Your spline file is ImageD11 specific")
            print("... from a fabio converted to edf first")
    elif options.format == 'GE':
        extn = ""
        # KE: This seems to be a mistake and keeps PeakSearch from working in
        # some cases.  Should be revisited if commenting it out causes problems.
#        options.ndigits = 0

    else:
        extn = options.format

    import fabio

    if options.interlaced:
        f0 = ["%s0_%04d%s"%(options.stem,i,options.format) for i in range(
                options.first,
                options.last+1)]
        f1 = ["%s1_%04d%s"%(options.stem,i,options.format) for i in range(
                options.first,
                options.last+1)]

        if options.iflip:
            f1 = [a for a in f1[::-1]]
   
        def fso(f0,f1):
            for a,b in zip(f0,f1):
                try:
                    yield fabio.open(a)
                    yield fabio.open(b)
                except:
                    print(a,b)
                    raise
        file_series_object = fso(f0,f1)     
        first_image = openimage( f0[0] )

    else:
        file_name_object = fabio.filename_object(
                options.stem,
                num = options.first,
                extension = extn,
                digits = options.ndigits)
                
        # Output files:

        first_image = openimage( file_name_object )
        import fabio.file_series
    # Use traceback = True for debugging
        file_series_object = fabio.file_series.new_file_series(
            first_image,
            nimages = options.last - options.first + 1,
            traceback = True  )
    
    if options.outfile[-4:] != ".spt":
        options.outfile = options.outfile + ".spt"
        print("Your output file must end with .spt, changing to ",options.outfile)

    # Omega overrides

    # global OMEGA, OMEGASTEP, OMEGAOVERRIDE
    OMEGA = options.OMEGA
    OMEGASTEP = options.OMEGASTEP
    OMEGAOVERRIDE = options.OMEGAOVERRIDE 
    # Make a blobimage the same size as the first image to process


    # List comprehension - convert remaining args to floats 
    # must be unique list so go via a set
    thresholds_list = list( set( [float(t) for t in options.thresholds] ) )
    thresholds_list.sort()
        
    li_objs={} # label image objects, dict of

    
    s = first_image.data.shape # data array shape
    
    # Create label images
    for t in thresholds_list:
        # the last 4 chars are guaranteed to be .spt above
        mergefile="%s_t%d.flt"%(options.outfile[:-4], t)
        spotfile = "%s_t%d.spt"%(options.outfile[:-4], t)
        li_objs[t]=labelimage(shape = s, 
                              fileout = mergefile, 
                              spatial = corrfunc,
                              sptfile=spotfile) 
        print("make labelimage",mergefile,spotfile)
    # Not sure why that was there (I think if glob was used)
    # files.sort()
    if options.dark is not None:
        print("Using dark (background)",options.dark)
        darkimage= openimage(options.dark).data.astype(numpy.float32)
    else:
        darkimage=None
    if options.darkoffset!=0:
        print("Adding darkoffset",options.darkoffset)
        if darkimage is None:
            darkimage = options.darkoffset
        else:
            darkimage += options.darkoffset
    if options.flood is not None:
        floodimage=openimage(options.flood).data
        cen0 = int(floodimage.shape[0]/6)
        cen1 = int(floodimage.shape[0]/6)
        middle = floodimage[cen0:-cen0, cen1:-cen1]
        nmid = middle.shape[0]*middle.shape[1]
        floodavg = numpy.mean(middle)
        print("Using flood",options.flood,"average value",floodavg)
        if floodavg < 0.7 or floodavg > 1.3:
            print("Your flood image does not seem to be normalised!!!")
         
    else:
        floodimage=None
    start = time.time()
    print("File being treated in -> out, elapsed time")
    # If we want to do read-ahead threading we fill up a Queue object with data 
    # objects
    # THERE MUST BE ONLY ONE peaksearching thread for 3D merging to work
    # there could be several read_and_correct threads, but they'll have to get the order right,
    # for now only one
    if options.oneThread:
        # Wrap in a function to allow profiling (perhaps? what about globals??)
        def go_for_it(file_series_object, darkimage, floodimage, 
                      corrfunc , thresholds_list , li_objs,
                      OMEGA, OMEGASTEP, OMEGAOVERRIDE ):
            for data_object in file_series_object:
                t = timer()
                if not isinstance( data_object, fabio.fabioimage.fabioimage ):
                    # Is usually an IOError
                    if isinstance( data_object[1], IOError):
                        sys.stdout.write(data_object[1].strerror  + '\n')
                                         # data_object[1].filename
                    else:
                        import traceback
                        traceback.print_exception(data_object[0],data_object[1],data_object[2])
                        sys.exit()
                    continue
                filein = data_object.filename
                if OMEGAOVERRIDE or "Omega" not in data_object.header:
                    data_object.header["Omega"] = OMEGA
                    OMEGA += OMEGASTEP
                    OMEGAOVERRIDE = True # once you do it once, continue
                if not OMEGAOVERRIDE and options.omegamotor != "Omega":
                    data_object.header["Omega"] = float( data_object.header[options.omegamotor] )
                data_object = correct( data_object, darkimage, floodimage,
                                       do_median = options.median,
                                       monitorval = options.monitorval,
                                       monitorcol = options.monitorcol,
                                       )
                t.tick(filein+" io/cor")
                peaksearch( filein, data_object , corrfunc , 
                            thresholds_list , li_objs )
            for t in thresholds_list:
                li_objs[t].finalise()
        go_for_it(file_series_object, darkimage, floodimage, 
                  corrfunc , thresholds_list, li_objs,
                  OMEGA, OMEGASTEP, OMEGAOVERRIDE )

            
    else:
        print("Going to use threaded version!")
        try:
            # TODO move this to a module ?

                
            class read_only(ImageD11_thread.ImageD11_thread):
                def __init__(self, queue, file_series_obj , myname="read_only",
                             OMEGA=0, OMEGAOVERRIDE = False, OMEGASTEP = 1):
                    """ Reads files in file_series_obj, writes to queue """
                    self.queue = queue 
                    self.file_series_obj = file_series_obj
                    self.OMEGA = OMEGA
                    self.OMEGAOVERRIDE = OMEGAOVERRIDE
                    self.OMEGASTEP = OMEGASTEP
                    ImageD11_thread.ImageD11_thread.__init__(self , 
                                                             myname=myname)
                    print("Reading thread initialised", end=' ')

                    
                def ImageD11_run(self):
                    """ Read images and copy them to self.queue """
                    for data_object in self.file_series_obj:
                        if self.ImageD11_stop_now():
                            print("Reader thread stopping")
                            break
                        if not isinstance( data_object, fabio.fabioimage.fabioimage ):
                            # Is usually an IOError
                            if isinstance( data_object[1], IOError):
#                                print data_object
#                                print data_object[1]
                                sys.stdout.write(str(data_object[1].strerror) + '\n')
#                                  ': ' + data_object[1].filename + '\n')
                            else:
                                import traceback
                                traceback.print_exception(data_object[0],data_object[1],data_object[2])
                                sys.exit()
                            continue
                        ti = timer()
                        filein = data_object.filename + "[%d]"%( data_object.currentframe )
                        try:
                            if self.OMEGAOVERRIDE:
                                # print "Over ride due to option",self.OMEGAOVERRIDE
                                data_object.header["Omega"] = self.OMEGA
                                self.OMEGA += self.OMEGASTEP
                            else:
                                if options.omegamotor != 'Omega' and options.omegamotor in data_object.header:
                                    data_object.header["Omega"] = float(data_object.header[options.omegamotor])
                                if "Omega" not in data_object.header:
                                    # print "Computing omega as not in header"
                                    data_object.header["Omega"] = self.OMEGA
                                    self.OMEGA += self.OMEGASTEP
                                    self.OMEGAOVERRIDE = True
                            # print "Omega = ", data_object.header["Omega"],data_object.filename
                        except KeyboardInterrupt:
                            raise
                        except:
                            continue
                        ti.tick(filein)
                        self.queue.put((filein, data_object) , block = True)
                        ti.tock(" enqueue ")
                        if self.ImageD11_stop_now():
                            print("Reader thread stopping")
                            break
                        
                    # Flag the end of the series
                    self.queue.put( (None, None) , block = True)

            class correct_one_to_many(ImageD11_thread.ImageD11_thread):
                def __init__(self, queue_read, queues_out,  thresholds_list,
                             dark = None , flood = None, myname="correct_one",
                             monitorcol = None, monitorval = None,
                             do_median = False):
                    """ Using a single reading queue retains a global ordering
                    corrects and copies images to output queues doing
                    correction once """
                    self.queue_read = queue_read
                    self.queues_out = queues_out 
                    self.dark = dark
                    self.flood = flood
                    self.do_median = do_median
                    self.monitorcol = monitorcol
                    self.monitorval = monitorval
                    self.thresholds_list = thresholds_list
                    ImageD11_thread.ImageD11_thread.__init__(self , 
                                                             myname=myname)
                    
                def ImageD11_run(self):
                    while not self.ImageD11_stop_now():
                        ti = timer()
                        filein, data_object = self.queue_read.get(block = True)
                        if filein is None:
                            for t in self.thresholds_list:
                                self.queues_out[t].put( (None, None) ,
                                                        block = True)
                            # exit the while 1
                            break 
                        data_object = correct(data_object, self.dark,
                                              self.flood,
                                              do_median = self.do_median,
                                              monitorval = self.monitorval,
                                              monitorcol = self.monitorcol,
                                              )
                        ti.tick(filein+" correct ")
                        for t in self.thresholds_list:
                            # Hope that data object is read only
                            self.queues_out[t].put((filein, data_object) ,
                                                   block = True)
                        ti.tock(" enqueue ")
                    print("Corrector thread stopping")



            class peaksearch_one(ImageD11_thread.ImageD11_thread):
                def __init__(self, q, corrfunc, threshold, li_obj,
                             myname="peaksearch_one" ):
                    """ This will handle a single threshold and labelimage
                    object """
                    self.q = q
                    self.corrfunc = corrfunc
                    self.threshold = threshold
                    self.li_obj = li_obj
                    ImageD11_thread.ImageD11_thread.__init__(
                        self,
                        myname=myname+"_"+str(threshold))


                def run(self):
                    while not self.ImageD11_stop_now():
                        filein, data_object = self.q.get(block = True)
                        if not isinstance( data_object, fabio.fabioimage.fabioimage ):
                            break
                        peaksearch( filein, data_object , self.corrfunc , 
                                    [self.threshold] , 
                                    { self.threshold : self.li_obj } )  
                    self.li_obj.finalise()

            # 8 MB images - max 40 MB in this queue
            read_queue = queue.Queue(5)
            reader = read_only(read_queue, file_series_object,
                               OMEGA = OMEGA,
                               OMEGASTEP = OMEGASTEP,
                               OMEGAOVERRIDE = OMEGAOVERRIDE )
            reader.start()
            queues = {}
            searchers = {}
            for t in thresholds_list:
                print("make queue and peaksearch for threshold",t)
                queues[t] = queue.Queue(3)
                searchers[t] = peaksearch_one(queues[t], corrfunc, 
                                              t, li_objs[t] )
            corrector = correct_one_to_many( read_queue,
                                             queues,
                                             thresholds_list,
                                             dark=darkimage,
                                             flood=floodimage,
                                             do_median = options.median,
                                             monitorcol = options.monitorcol,
                                             monitorval = options.monitorval)
            corrector.start()
            my_threads = [reader, corrector]
            for t in thresholds_list[::-1]:
                searchers[t].start()
                my_threads.append(searchers[t])
            nalive = len(my_threads)
            def empty_queue(q):
                while 1:
                    try:
                        q.get(block=False, timeout=1)
                    except:
                        break
                q.put((None, None), block=False)
            while nalive > 0:
                try:
                    nalive = 0
                    for thr in my_threads:
                        if thr.isAlive():
                            nalive += 1
                    if options.killfile is not None and \
                           os.path.exists(options.killfile):
                        raise KeyboardInterrupt()
                    time.sleep(1)
                except KeyboardInterrupt:
                    print("Got keyboard interrupt in waiting loop")
                    ImageD11_thread.stop_now = True
                    try:
                        time.sleep(1)
                    except:
                        pass
                    empty_queue(read_queue)
                    for t in thresholds_list:
                        q = queues[t]
                        empty_queue(q)
                    for thr in my_threads:
                        if thr.isAlive():
                            thr.join(timeout=1)                        
                    print("finishing from waiting loop")
                except:
                    print("Caught exception in waiting loop")
                    ImageD11_thread.stop_now = True
                    time.sleep(1)
                    empty_queue(read_queue)
                    for t in thresholds_list:
                        q = queues[t]
                        empty_queue(q)
                    for thr in my_threads:
                        if thr.isAlive():
                            thr.join(timeout=1)
                    raise
                    

        except ImportError:
            print("Probably no threading module present")
            raise
    


def get_options(parser):
        """ Add our options to a parser object """
        parser.add_argument("-n", "--namestem", action="store",
            dest="stem", type=str, default="data",
            help="Name of the files up the digits part  "+\
                 "eg mydata in mydata0000.edf" )
        parser.add_argument("-F", "--format", action="store",
            dest="format",default=".edf", type=str,
            help="Image File format, eg edf or bruker" )
        parser.add_argument("-f", "--first", action="store",
            dest="first", default=0, type=int,
            help="Number of first file to process, default=0")
        parser.add_argument("-l", "--last", action="store",
            dest="last", type=int,default =0,
            help="Number of last file to process")
        parser.add_argument("-o", "--outfile", action="store",
            dest="outfile",default="peaks.spt", type=str,
            help="Output filename, default=peaks.spt")
        parser.add_argument("-d", "--darkfile", action="store",
            dest="dark", default=None,  type=ImageD11options.ImageFileType(mode='r'),
            help="Dark current filename, to be subtracted, default=None")
        dod = 0
        parser.add_argument("-D", "--darkfileoffset", action="store",
            dest="darkoffset", default=dod, type=float,
            help=
         "Constant to subtract from dark to avoid overflows, default=%d"%(dod))
        # s="/data/opid11/inhouse/Frelon2K/spatial2k.spline"
        parser.add_argument("-s", "--splinefile", action="store",
            dest="spline", default=None, type=ImageD11options.SplineFileType(mode='r'),
            help="Spline file for spatial distortion, default=None" )
        parser.add_argument("-p", "--perfect_images", action="store",
               choices=["Y","N"], default="Y", dest="perfect",
                          help="Ignore spline Y|N, default=N")
        parser.add_argument("-O", "--flood", action="store", 
                            type=ImageD11options.ImageFileType(mode='r'),
                          default=None, dest="flood",
                          help="Flood file, default=None")
        parser.add_argument("-t", "--threshold", action="append", type=float,
             dest="thresholds", default=None,
             help="Threshold level, you can have several")
        parser.add_argument("--OmegaFromHeader", action="store_false",
                          dest="OMEGAOVERRIDE", default=False, 
                          help="Read Omega values from headers [default]")
        parser.add_argument("--OmegaOverRide", action="store_true",
                          dest="OMEGAOVERRIDE", default=False, 
                          help="Override Omega values from headers")
        parser.add_argument("--singleThread", action="store_true",
                          dest="oneThread", default=False, 
                          help="Do single threaded processing")
        # if you want to do this then instead I think you want
        # python -m cProfile -o xx.prof peaksearch.py ...
        # python -m pstats xx.prof
        #    ... sort
        #    ... stats
#        parser.add_argument("--profile", action="store", 
#                        type=ImageD11options.ProfilingFileType,
#                          dest="profile_file", default=None, 
# help="Write profiling information (you will want singleThread too)")
        parser.add_argument("-S","--step", action="store",
                          dest="OMEGASTEP", default=1.0, type=float,
 help="Step size in Omega when you have no header info")
        parser.add_argument("-T","--start", action="store",
                          dest="OMEGA", default=0.0, type=float,
 help="Start position in Omega when you have no header info")
        parser.add_argument("-k","--killfile", action="store",
                          dest="killfile", default=None, 
                          type=ImageD11options.FileType(),
 help="Name of file to create stop the peaksearcher running")
        parser.add_argument("--ndigits", action="store", type=int,
                dest = "ndigits", default = 4,
                help = "Number of digits in file numbering [4]")
        parser.add_argument("-P", "--padding", action="store",
             choices=["Y","N"], default="Y", dest="padding",
                          help="Is the image number to padded Y|N, e.g. "\
                    "should 1 be 0001 or just 1 in image name, default=Y")        
        parser.add_argument("-m", "--median1D", action="store_true",
               default=False, dest="median",
               help="Computes the 1D median, writes it to file .bkm and" \
               +" subtracts it from image. For liquid background"\
               +" on radially transformed images")
        parser.add_argument("--monitorcol", action="store", type=str,
                           dest="monitorcol",
                           default = None,
                           help="Header value for incident beam intensity")
        parser.add_argument("--monitorval", action="store", type=float,
                          dest="monitorval",
                          default = None,
                          help="Incident beam intensity value to normalise to")
        parser.add_argument("--omega_motor", action="store", type=str,
                          dest = "omegamotor", default = "Omega",
           help = "Header value to use for rotation motor position [Omega]")
        parser.add_argument("--omega_motor_step", action="store", type=str,
                          dest = "omegamotorstep", default = "OmegaStep",
           help = "Header value to use for rotation width [OmegaStep]")
        parser.add_argument("--interlaced", action="store_true", 
                          dest = "interlaced", default = False,
           help = "Interlaced DCT scan")
        parser.add_argument("--iflip", action="store_true",
                          dest="iflip", default=False,
                          help = "Reverse second half of interlaced scan")
        return parser





def get_help(usage = True):
    """ return the help string for online help """
    try:
        import StringIO as io
    except:
        # python3
        import io
    import argparse
    if usage:
        o = get_options(argparse.ArgumentParser())
    else:
        o = get_options(argparse.ArgumentParser(usage=argparse.SUPPRESS))
    f = io.StringIO()
    o.print_help(f)
    return f.getvalue()


if __name__=="__main__":
    raise Exception("Please use the driver script peaksearch.py")





## Automatically adapted for numpy.oldnumeric Sep 06, 2007 by alter_code1.py


from __future__ import print_function


# ImageD11_v0.4 Software for beamline ID11
# Copyright (C) 2005  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import numpy as np, math
import ImageD11.indexing
import xfab.tools

# helpers : put these into xfab.tools at some point?
def e6_to_symm(e):
    """ Follows the eps convention from xfab 
    eps = [e11, e12, e13, e22, e23, e33]
    """
    e11, e12, e13, e22, e23, e33 = e
    return np.array(((e11,e12,e13),
                     (e12,e22,e23),
                     (e13,e23,e33)))

def symm_to_e6(m):
    """ Follows the eps convention from xfab 
    eps = [e11, e12, e13, e22, e23, e33]
    """
    return np.array( ( m[0,0], m[0,1], m[0,2],
                               m[1,1], m[1,2],
                                       m[2,2] ) )



class grain:
    def __init__(self,ubi,translation=None, **kwds):
        if translation is None:
            # If translation has not been read from ubi file make it 
            # be None to avoid confusion with a grain which is known
            # to be at [0,0,0]
            self.translation = None
        else:
            self.translation = np.array(translation,np.float)
        self.set_ubi(ubi)

    def set_ubi(self, ubi):
        """ Update the orientation and clear cached values """
        self.ubi = np.array(ubi,np.float)
        self.clear_cache()        

    def clear_cache(self):
        # We will cache a bunch of things and access them
        # via properties. Usually we return copies of these
        # so we avoid people overwriting cached values by accident.
        # e.g. cell = grain.unitcell
        #      cell[4:] = 90 # degree angle for orthorhombic
        #      oups : grain._unitcell was corrupted!
        self._ub = None
        self._u = None
        self._rod = None
        self._mt = None
        self._rmt = None
        self._unitcell = None

    @property 
    def ub(self):
        """ The UB matrix from Busing and Levy
        columns are the reciprocal space lattice vectors
        """
        if self._ub is None:
            self._ub = np.linalg.inv(self.ubi)
        return self._ub.copy()

    @property
    def u(self):
        """ The orientation matrix (U) from Busing and Levy """
        if self._u is None:
            self._u = xfab.tools.ubi_to_u(self.ubi)
        return self._u.copy()

    @property
    def Rod(self):
        """ A Rodriguez vector. 
        Length proportional to angle, direction is axis"""
        if self._rod is None:
            self._rod = xfab.tools.u_to_rod( self.u )
        return self._rod.copy()

    @property
    def mt(self):
        """Metric tensor """
        if self._mt is None:
            self._mt = np.dot( self.ubi, self.ubi.T )
        return self._mt.copy()

    @property
    def rmt(self):
        """ Reciprocal metric tensor """
        if self._rmt is None:
            self._rmt = np.linalg.inv( self.mt )
        return self._rmt.copy()
        
    @property
    def unitcell(self):
        """ a,b,c,alpha,beta,gamma """
        if self._unitcell is None:
            self._unitcell = xfab.tools.ubi_to_cell( self.ubi )
        return self._unitcell.copy()

    def eps_grain(self, dzero_cell):
        """ dzero_cell = [a,b,c,alpha,beta,gamma]
        Returns eps = [e11, e12, e13, e22, e23, e33]
        ... in the grain system
        """
        B_xfab = xfab.tools.form_b_mat( self.unitcell )
        eps = xfab.tools.b_to_epsilon( B_xfab, dzero_cell )
        return eps       

    def eps_sample_matrix(self, dzero_cell):
        """ dzero_cell = [a,b,c,alpha,beta,gamma]
        Returns eps as a symmetric matrix
        ... in the sample system (z usually up axis, x along the beam at omega=0)
        """
        Eg = self.eps_grain_matrix( dzero_cell )
        U = self.u
        Es = np.dot( U,  np.dot( Eg, U.T ) )
        return Es

    def eps_grain_matrix(self, dzero_cell):
        """ dzero_cell = [a,b,c,alpha,beta,gamma]
        Returns eps as a symmetric matrix
        ... in the grain system 
        """
        eps = self.eps_grain( dzero_cell )
        return e6_to_symm( eps )

    def eps_sample(self, dzero_cell):
        """ dzero_cell = [a,b,c,alpha,beta,gamma]
        Returns eps as [e11, e12, e13, e22, e23, e33]
        ... in the sample system (z usually up axis, x along the beam at omega=0)
        """
        Es = self.eps_sample_matrix( dzero_cell )
        return symm_to_e6( Es )

        


    
def write_grain_file(filename, list_of_grains):
    f = open(filename, "w")
    for g in list_of_grains:
        t = g.translation
        f.write("#translation: %g %g %g\n"%(t[0],t[1],t[2]))
        if hasattr(g,"name"):
            f.write("#name %s\n"%(g.name.rstrip()))
        if hasattr(g,"intensity_info"):
            f.write("#intensity_info %s\n"%(g.intensity_info.rstrip()))
        if hasattr(g,"npks"):
            f.write("#npks %d\n"%(int(g.npks)))
        if hasattr(g,"nuniq"):
            f.write("#nuniq %d\n"%(int(g.nuniq)))
        if hasattr(g,"Rod"):
            try:
                f.write("#Rod %f %f %f\n"%tuple([float(r) for r in g.Rod]))
            except:
                f.write("#Rod %s"%(g.Rod))
        f.write("#UBI:\n")
        u = g.ubi
        # More than float32 precision
        f.write("%.9g %.9g %.9g\n"  %(u[0,0],u[0,1],u[0,2]))
        f.write("%.9g %.9g %.9g\n"  %(u[1,0],u[1,1],u[1,2]))
        f.write("%.9g %.9g %.9g\n\n"%(u[2,0],u[2,1],u[2,2]))
    f.close()

def read_grain_file(filename):
    """read ubifile and return a list of ubi arrays """
    f = open(filename, "r")
    grainsread = []
    u = []
    t = None
    p = {}
    for line in f:
        if line.find("#translation:")==0:
            t = [ float(x) for x in line.split()[1:]]
            continue
        if line[0] == "#" and line.find("UBI")<0:
            k,v=line[1:].split(" ",1)
            p[k]=v
            continue
        if line[0] == "#" and line.find("intensity_info")>-1:
            p["intensity_info"] = line.split("intensity_info")[1].rstrip()
        if line.find("#")==0: continue
        vals = [ float(x) for x in line.split() ]
        if len(vals) == 3:
            u = u + [vals]
        if len(u)==3:
            grainsread.append( grain(u, t) )
            for k in ["name","npks","nuniq","intensity_info"]: # Rod - is recomputed when needed
                if k in p:
                    setattr(grainsread[-1], k, p[k])
            p={}
            u = []
            t = None
    f.close()
    return grainsread




from __future__ import print_function

# Get Strain/Stress from ImageD11 UBI/map files
# Copyright (C) 2015 Younes ELHACHI
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import numpy as np
import math
from ImageD11 import transform, unitcell, columnfile
from ImageD11.parameters import par, parameters
from ImageD11.grain import read_grain_file
try:
    from FitAllB.conversion import grain2sample, strain2stress,formStiffnessMV
except:
    print( "You need to install FitAllB")
    print( "You will get error messages if you try to compute strain!")
from xfab.tools import ubi_to_u_and_eps,ubi_to_cell

def readubis(ubifile):
    """read ubifile and return a list of ubi arrays """
    f = open(ubifile, "r")
    ubisread = []
    u = []
    for line in f:
        if line[0]=="#":
            continue
        vals = [ float(x) for x in line.split() ]
        if len(vals) == 3:
            u = u + [vals]
        if len(u)==3:
            ubisread.append(np.array(u))
            u = []
    f.close()
    return ubisread

def write_ubi_file(filename, ubilist):
    """ save 3x3 matrices into file """
    f=open(filename,"w")
    for u in ubilist:
        f.write("%f %f %f\n"  %(u[0][0],u[0][1],u[0][2]))
        f.write("%f %f %f\n"  %(u[1][0],u[1][1],u[1][2]))
        f.write("%f %f %f\n\n"%(u[2][0],u[2][1],u[2][2]))
    f.close() 



class solver:
    """
    A class for getting strain and stress tensors
    """
    def __init__(self,
            unitcell=None,
            ubis=None,
            crystal_symmetry=None,
            c11=None,c12=None,c13=None,c14=None,c15=None,c16=None,
            c22=None,c23=None,c24=None,c25=None,c26=None,
            c33=None,c34=None,c35=None,c36=None,
            c44=None,c45=None,c46=None,
            c55=None,c56=None,
            c66=None):
        """
        unitcell would be a list of six elements [a, b, c, alpha, beta, gamma]
        ubis would be a list of orientation matrices as by ImageD11 convention
        symmetry would be isotropic, cubic, tetragonal_high... (see FitAllB.conversion) to form the stiffness tensor C
        The rest of the arguments are parameters.
        """
        
        self.cell__a = None
        self.cell__b = None
        self.cell__c = None
        self.cell_alpha = None
        self.cell_beta = None
        self.cell_gamma = None
        if unitcell is not None:
            if len(unitcell)==6:
                self.cell__a = unitcell[0]
                self.cell__b = unitcell[1]
                self.cell__c = unitcell[2]
                self.cell_alpha = unitcell[3]
                self.cell_beta = unitcell[4]
                self.cell_gamma = unitcell[5]
            else:
                raise Exception("The unit cell must be defined by six parameters!")
        self.ubis=ubis
        self.crystal_symmetry=crystal_symmetry
        self.c11=c11
        self.c12=c12
        self.c13=c13
        self.c14=c14
        self.c15=c15
        self.c16=c16
        self.c22=c22
        self.c23=c23
        self.c24=c24
        self.c25=c25
        self.c26=c26
        self.c33=c33
        self.c34=c34
        self.c35=c35
        self.c36=c36
        self.c44=c44
        self.c45=c45
        self.c46=c46
        self.c55=c55
        self.c56=c56
        self.c66=c66
        self.parameterobj = parameters(cell__a=self.cell__a,
                                       cell__b=self.cell__b,
                                       cell__c=self.cell__c,
                                       cell_alpha=self.cell_alpha,
                                       cell_beta=self.cell_beta,
                                       cell_gamma=self.cell_gamma,                                                 
                                       crystal_symmetry=self.crystal_symmetry,
                                       c11=self.c11,c12=self.c12,c13=self.c13,c14=self.c14,c15=self.c15,c16=self.c16,
                                       c22=self.c22,c23=self.c23,c24=self.c24,c25=self.c25,c26=self.c26,
                                       c33=self.c33,c34=self.c34,c35=self.c35,c36=self.c36,
                                       c44=self.c44,c45=self.c45,c46=self.c46,
                                       c55=self.c55,c56=self.c56,
                                       c66=self.c66)
        self.epsilon=[]
        self.sigma=[]

        
    def loadmap(self,filename):
        try:
            self.map=read_grain_file(filename)
            self.ubis=[x.ubi for x in self.map]
        except:
            print("error when reading %s\n",filename)
            raise
    
    
    def loadpars(self,filename=None):
        if filename is not None:
            self.parameterobj.loadparameters(filename)
        self.parameterobj.update_other(self)
        #update also the unitcell list (because the element are included in parameterobj but not the list):
        #self.unitcell=[self.cell__a, self.cell__b, self.cell__c, self.cell_alpha, self.cell_beta, self.cell_gamma]

    def updateparameters(self):
        self.savepars()
        self.pars=self.parameterobj.parameters
        #update also the unitcell list (because the element are included in parameterobj but not the list):
        #self.unitcell=[self.cell__a, self.cell__b, self.cell__c, self.cell_alpha, self.cell_beta, self.cell_gamma]

    def savepars(self,filename=None):
        self.parameterobj.update_yourself(self)
        if filename is not None:
            self.parameterobj.saveparameters(filename)
    
        
    def unitcell(self):
        return [self.cell__a, self.cell__b, self.cell__c, self.cell_alpha, self.cell_beta, self.cell_gamma]
    
    def setunitcell(self,uc):
        """    this is used to set all the unit cell elements in one shot    """
        self.cell__a = uc[0]
        self.cell__b = uc[1]
        self.cell__c = uc[2]
        self.cell_alpha = uc[3]
        self.cell_beta = uc[4]
        self.cell_gamma = uc[5]
        
    
    def MVStiffness(self):
        return formStiffnessMV(crystal_system=self.crystal_symmetry,
                               c11=self.c11,c12=self.c12,c13=self.c13,c14=self.c14,c15=self.c15,c16=self.c16,
                               c22=self.c22,c23=self.c23,c24=self.c24,c25=self.c25,c26=self.c26,
                               c33=self.c33,c34=self.c34,c35=self.c35,c36=self.c36,
                               c44=self.c44,c45=self.c45,c46=self.c46,
                               c55=self.c55,c56=self.c56,
                               c66=self.c66)
          

    def compute_write_eps_sig(self,outputfile):
        """    Compute strain and stress in crystal and sample co-ordinates system    """
        
        if self.ubis is not None:
            
            writestress = True
            
            f = open(outputfile,'w')
            ''' the used parameters will be the header of the output file'''
            for k,v in sorted(self.parameterobj.parameters.items()):
                f.write(("%s %s\n")%(k,v))
            ''' write titles'''
            f.write("##############################################\n")
            f.write("cell__a cell__b cell__c cell_alpha cell_beta cell_gamma u11 u12 u13 u21 u22 u23 u31 u32 u33 ")
            f.write("eps11_c eps22_c eps33_c eps12_c eps13_c eps23_c eps11_s eps22_s eps33_s eps12_s eps13_s eps23_s ")
            f.write("sig11_c sig22_c sig33_c sig12_c sig13_c sig23_c sig11_s sig22_s sig33_s sig12_s sig13_s sig23_s\n")
            
            '''this is the part where we compute strain and stress'''
            for ubi in self.ubis:
                U, eps = ubi_to_u_and_eps(ubi, self.unitcell())
                #U, eps = ubi_to_u_and_eps(ubi, ubi_to_cell(self.ubis[0]))
                epsM = [ [ eps[0], eps[1], eps[2] ],        #write the strain tensor list as a matrix
                         [ eps[1], eps[3], eps[4] ],
                         [ eps[2], eps[4], eps[5] ] ]
                epsS = np.dot( U, np.dot( epsM, U.T ) )     #epsilon in sample co-ordinates
                sigM= np.empty((3,3))
                sigS= np.empty((3,3))
                try:
                    sigM = strain2stress( np.array(epsM), self.MVStiffness() )      #write the stress tensor as a symmetric matrix in crystal co-ordinates
                    sigS = np.dot( U, np.dot( sigM, U.T ) )     #sigma in sample co-ordinates
                except:
                    print("couldn't compute stress! please check the crystal_symmetry parameters and elastic constants")
                    writestress = False
                
                
                ''' writing down the results'''
                
                f.write(("%f "*6)%tuple(ubi_to_cell(ubi)))
                f.write(("%f "*9)%tuple(U.ravel()))
                ligne = ""
                for i,j in [(0,0),(1,1),(2,2),(0,1),(0,2),(1,2)]:
                    ligne = ligne + " " + str(100.*np.array(epsM)[i,j])
                for i,j in [(0,0),(1,1),(2,2),(0,1),(0,2),(1,2)]:
                    ligne = ligne + " " + str(100.*epsS[i,j])
                if writestress==True:
                    for i,j in [(0,0),(1,1),(2,2),(0,1),(0,2),(1,2)]:
                        ligne = ligne + " " + str(np.array(sigM)[i,j])
                    for i,j in [(0,0),(1,1),(2,2),(0,1),(0,2),(1,2)]:
                        ligne = ligne + " " + str(sigS[i,j])
                
                f.write(ligne[1:])
                f.write("\n")
            f.close()




"""
fast radial regrouping method proposed by Peter Boesecke
"""
import numpy as np, pylab as pl

# 2018 code:
#   compute radius / azimuth
#        integer bin for radius
#        sort on radial bin, then azimuth (or vice versa)
#
#   Re-order an input array to write output
#    no splits or sums n->n mapping
#

def compute_r_chi( dims, pars ):
    """
    Replace this with pyFAI / ImageD11 / whatever
      dims = array dimension
      pars = center in slow/fast
    returns r/chi (float)
    """
    i, j = np.mgrid[0:dims[0], 0:dims[1]]
    i = i - pars[0]
    j = j - pars[1]
    r = np.sqrt( i*i + j*j )
    c = np.arctan2( i, j )
    return r, c

dims = ( 2048, 2048 )
pars = 1022.1, 1018.5, 1.25
rad, chi = compute_r_chi( dims, pars )

irad = np.round( rad / pars[2] ).astype( int )
import time
start=time.time()
lut = np.lexsort( (chi.ravel(), irad.ravel() ) )
print(time.time()-start)
start=time.time()
radout = rad.ravel()[lut]
iradout = irad.ravel()[lut]
chiout = chi.ravel()[lut]
# this is where the pixels read from
adrout = np.arange(dims[0]*dims[1])[lut]
# required to save
#   length of radial rows, r, IR and chi value for each
# TODO: optimise the order to apply lut
#      assert len(set(lut)) == len(lut)


print(time.time()-start)



pl.subplot(221)
pl.imshow(rad)
pl.title("radius")
pl.colorbar()
pl.subplot(222)
pl.imshow(chi)
pl.title("azimuth")
pl.subplot(223)
pl.imshow(adrout.reshape(dims),aspect='auto',
          cmap='prism', interpolation='nearest')

pl.title("Source address to come from")
pl.subplot(224)
pl.imshow(np.argsort(lut).reshape(dims),aspect='auto',
          cmap='prism', interpolation='nearest')

pl.title("Output address to go to")
pl.show()




from __future__ import print_function
from six.moves import input
from ImageD11 import peakmerge, indexing, transformer
from ImageD11 import grain
import sys, os, numpy as np

mytransformer = transformer.transformer()
#
# Your work starts here:
#    
print("select only peaks of certain rings ")

mytransformer.loadfiltered( sys.argv[1] )
mytransformer.loadfileparameters(  sys.argv[2] )
outfile = sys.argv[3]

mytransformer.updateparameters( )
tth, eta = mytransformer.compute_tth_eta( )

from matplotlib.pylab import plot, show, figure
figure(1)
plot(tth,eta,",")

mytransformer.colfile.filter( mytransformer.colfile.Number_of_pixels > 3 )        
tth =    mytransformer.colfile.tth

mytransformer.addcellpeaks()

rh = mytransformer.unitcell.ringhkls
peaks = list(rh.keys())
peaks.sort()
m = 0
print("Ring ds  tth  (h k l) mult  npks  npks/mult")
for i,d in enumerate(peaks):
    thistth = mytransformer.theorytth[i]
    mult = len(rh[d])
    print("Ring %3d %2.4f %8.4f"%(i,d,thistth),"(%2d %2d %2d)"%rh[d][0],"%2d"%(mult), end=' ')
    msk = ((thistth+0.2) > tth)&(tth > (thistth-0.2))
    npk = msk.sum()
    print("%8d %8d"%(npk,1.0*npk/mult))
    m += msk.sum()
print(m, tth.shape)

figure(2)
plot(tth,np.log(mytransformer.colfile.sum_intensity),  ",")
show()

if 1:
    while 1:
        rings_to_use = input("Enter comma separated list of rings to use")
        try:
            rs = [int(v) for v in rings_to_use.split(',')]
        except:
            print("I don't understand, please try again")
            continue
        print("You entered"," ".join([str(r) for r in rs]))
        if input("OK?") in ['Y','y']:
            break
else:
    rs = [ 0, 2, 6, 10]

thistth = mytransformer.theorytth[rs[0]]
msk = ((thistth+0.2) > tth)&(tth > (thistth-0.2))
for r in rs[1:]:
    thistth = mytransformer.theorytth[r]
    msk |= ((thistth+0.2) > tth)&(tth > (thistth-0.2))

mytransformer.colfile.filter(msk)

mytransformer.write_colfile(sys.argv[3])
t,e=mytransformer.compute_tth_eta( )
figure(1)
plot(t,e,"o")
show()




"""
Script to put the menus together and build an appli with each bit
being relatively clutterfree

Each of guiindexer, guipeaksearch and guitransformer offer members
menuitems to put in menus. The overall gui (imaged11_gui) inherits
from this I think

# The code was copied from a book somewhere
# probably programming python or maybe cookbook.
# Most probably programming python by Mark Lutz
"""
try:
    import Tkinter as Tk
except:
    # python 3 ? 
    import tkinter as Tk

class GuiMaker(Tk.Frame):   # Inherit from Tk frame
    """
    You must inherit from this class and implement the start and makeWidgets
    methods
    """
    menuBar=[]
    def __init__(self,parent=None):
        Tk.Frame.__init__(self,parent)
        self.pack(expand=Tk.YES, fill=Tk.BOTH)
        import ImageD11
        self.statusbar=Tk.Label(self,text="Welcome to ImageD11 version "+ImageD11.__version__)
        self.statusbar.pack(side=Tk.BOTTOM)
        self.start()
        self.makeWidgets()
        self.makeMenuBar()

    def makeMenuBar(self):
        menubar = Tk.Menu(self.master,tearoff=0)
        self.master.config(menu=menubar)
        for (name, key, items) in self.menuBar:
            pulldown = Tk.Menu(menubar)
            self.addMenuItems(pulldown,items)
            menubar.add_cascade(label=name, underline=key, menu=pulldown)

    def addMenuItems(self, menu, items):
        for item in items:
            if item=="separator":
                menu.add_separator({})
            else:
                menu.add_command(label = item[0], underline = item[1], command=item[2] )





from numpy import dot, round_, array, float, allclose, asarray, fabs,\
    argmin, argmax, sqrt, argsort, take, sum, where, ndarray, eye,\
    zeros, cross
from numpy.linalg import inv, LinAlgError

from ImageD11.lattice_reduction import lattice, find_lattice
from ImageD11.rc_array import rc_array

def get_eu_gv():
    from ImageD11.indexing import indexer, ubitocellpars
    o = indexer()
    o.readgvfile("eu3.gve" , quiet = True)
    return o.gv

def test_fft():
    gv = get_eu_gv()
    from ImageD11.fft_index_refac import grid
    from ImageD11.indexing import ubitocellpars, write_ubi_file,\
        refine
    g = grid( np = 128,
              mr = 1.0,
              nsig = 20)
    g.gv_to_grid_new(gv)
    g.fft()
    g.props()
    g.peaksearch(open("eu.patterson_pks","w"))
    g.read_peaks("eu.patterson_pks")
    vecs = rc_array(g.UBIALL.T , direction='col')
    assert vecs.shape == (3, g.colfile.nrows)
    order = argsort( g.colfile.sum_intensity )[::-1]
    vecs = take( vecs, order, axis = 1)
    min_pks = 300
    ntry = 0

    assert g.gv.shape[1] == 3
    tv = rc_array( g.gv, direction='row' )
    print "Finding lattice l1 from patterson"
    l1 = find_lattice( vecs,
                       min_vec2 = 9,
                       test_vecs = tv,
                       n_try = 20 )
    print "r2c == ubi matrix"
    print l1.r2c 
    print "scores", l1.score( tv )
    print "cell",ubitocellpars(l1.r2c)
    l1.r2c = refine( l1.r2c, g.gv, tol=0.1)
    l1.c2r = inv(l1.r2c)
    print "With refine",l1.score( tv )
    print "cell",ubitocellpars(l1.r2c)
    print "Finding lattice l2 with gvectors to test"
    l2 = find_lattice( vecs,
                       min_vec2 = 9,
                       n_try = 20,
                       test_vecs = tv)
    print "r2c == ubi matrix"
    print l2.r2c       
    print "scores", l2.score (tv)
    print "cell",ubitocellpars(l2.r2c)
    l2.r2c = refine( l2.r2c, g.gv, tol=0.1)
    l2.c2r = inv(l2.r2c)
    print "With refine",l2.score( tv )
    print "cell",ubitocellpars(l2.r2c)
    return

def test_eu():
    #Conventional cell gives:
    #0 = (0, 1,  1)
    #1 = (0, 1, -1)
    #6 = (1, 0, -1)
    gv = get_eu_gv()
    v1 = gv[0]
    v2 = gv[1]
    v3 = gv[6]
    assert len(v1) == 3
    print v1,v2,v3
    # This means that the g-vectors are in row direction
    l = lattice ( v1, v2, v3, direction='row')
    esum = 0.0
    gv = rc_array( gv , direction='row' )
    # print v1, v2, v3
    print "ubi/r2c",l.r2c
    print "ub /c2r",l.c2r
    print "dot(l.r2c, gv[0])",dot(l.r2c, gv[0])
    for v in gv:
        #print ("%8.5f "*3+"%8.5f "*3)%tuple( list(v)+list(l.flip(v))),
        assert len(v) == 3
        err   = l.remainders( v )
        esum += sqrt(dot(err,err))
        #print "%.5f"%(sqrt(dot(err,err)))
    #import sys
    #sys.exit()
    # print 
    assert esum / len(gv) < 0.0102, "Did not fit"
    s = l.score(gv, tol = 0.1)
    assert  s == 583, "Expecting to index 582 peaks, got %s"%(s) 
    print "Indexing of eu3.gve, Average",esum / len(gv),"for",s,"peaks"
    print "UBI is",l.r2c

def test_eu_find():
    gv = get_eu_gv()
    vecs = rc_array(gv, direction="row")
    l = find_lattice( vecs,
                      # This next bit is important - tolerance for gve 
                      # is very different to patterson
                      min_vec2=1./81.,
                      n_try=20,
                      test_vecs=vecs)
    s = l.score(vecs, tol=0.1)
    print "Eu3 using find_lattice scores",s

def test2():
    """ Adding a fourth vector """
    o = lattice( array( [ 1, 0, 0] , float) ,
                 array( [ 0, 1, 0] , float) ,
                 array( [ 0, 0, 2] , float) , direction = 'row')
    # ... how to do it?
    #print o.r2c
    #print o.c2r
    u = rc_array( [ 0, 0, 1] , dtype=float, direction = 'row') 
    r  = o.remainders( u )
    assert ( r == array( [ 0, 0, 1] , float) ).all()
    o = o.withvec( r )
    assert allclose(o.c2r[0], array( [ 1, 0, 0] , float) )
    assert allclose(o.c2r[1], array( [ 0, 1, 0] , float) )
    assert allclose(o.c2r[2], array( [ 0, 0, 1] , float) )
    assert allclose(o.c2r, o.r2c)
    
def test1():
    """ Make a lattice from 3 vectors"""
    o = lattice( array( [ 1, 0, 0] , float) ,
                 array( [ 2, 1, 0] , float) ,
                 array( [ 3, 4, 1] , float) , direction = 'col')
    assert allclose(o.c2r[:,0], array( [ 1, 0, 0] , float) )
    assert allclose(o.c2r[:,1], array( [ 0, 1, 0] , float) )
    assert allclose(o.c2r[:,2], array( [ 0, 0, 1] , float) )
    # Both are identity
    assert allclose(o.c2r, eye(3))
    assert allclose(o.r2c, eye(3))

    """ 3 more difficult vectors """
    o = lattice( array( [ 1, 0, 1] , float) ,
                 array( [ 0, 1, 0] , float) ,
                 array( [ 0, 0, 1] , float) , direction = 'col')
    assert allclose(o.c2r[:,0], array( [ 1, 0, 0] , float) ), str(o.c2r)
    assert allclose(o.c2r[:,1], array( [ 0, 1, 0] , float) ), str(o.c2r)
    assert allclose(o.c2r[:,2], array( [ 0, 0, 1] , float) ), str(o.c2r)
    assert allclose(o.c2r, eye(3))
    assert allclose(o.r2c, eye(3))
    

    """ 3 shorter vectors """
    # print '3 short'
    o = lattice( array( [1e-2,  0,  0] , float) ,
                 array( [  0,1e-2,  0] , float) ,
                 array( [  0,  0,1e-2] , float) , direction='row' )
    # Consider those as 3 g-vectors
    # supplied rows go into c2r matrix as columns
    # The r2c matrix (==ubi) is then the inverse of this
    assert allclose(o.r2c[:,0], array( [ 100, 0, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,1], array( [ 0, 100, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,2], array( [ 0, 0, 100] , float) ), str(o.r2c)

    # g-vectors written as rows here - note transpose
    g = array( [[1e-2,   0.,0.], 
                [1e-2, 1e-2,0.], 
                [1e-2,   0.,1e-2],
                [  0.,   0.,1e-2]])
    #print g, g.shape
    g = rc_array( g , direction='row')
    assert g.shape == ( 4, 3)
    g.check()
    #print g.shape
    #print g
    hkl = o.flip( g ) 
    assert hkl.direction == 'col', hkl.direction
    # print g,"\nFlips to give\n",hkl
    assert allclose(hkl, 
                    rc_array(array([[1.,0.,0.],
                                    [1.,1.,0.],
                                    [1.,0.,1.],
                                    [0.,0.,1.]]).T,
                             direction='col') ), str(hkl)


    # print '3 long'
    """ 3 longer vectors """
    o = lattice( array( [ 10,  0,  0] , float) ,
                 array( [  0, 10,  0] , float) ,
                 array( [  0,  0, 10] , float) , direction='col' )
    assert allclose(o.r2c[:,0], array( [ 10, 0, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,1], array( [ 0, 10, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,2], array( [ 0, 0, 10] , float) ), str(o.r2c)
    test_col_vec = rc_array([10.,10.,10.], direction='col') 
    flipped = o.flip( test_col_vec )
    assert flipped.direction == 'row'
    assert allclose( flipped,
                     rc_array([1.,1.,1.], direction='col') ), str(flipped)

    """ 3 shorter vectors row"""
    # print '3 short'
    o = lattice( array( [1e-2,  0,  0] , float) ,
                 array( [  0,1e-2,  0] , float) ,
                 array( [  0,  0,1e-2] , float) , direction='row' )
    # Consider those as 3 g-vectors
    assert allclose(o.r2c[:,0], array( [ 100, 0, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,1], array( [ 0, 100, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,2], array( [ 0, 0, 100] , float) ), str(o.r2c)
    g = rc_array([[1e-2,   0.,0.], 
                  [1e-2, 1e-2,0.], 
                  [1e-2,   0.,1e-2],
                  [  0.,   0.,1e-2]],
                 direction='row')
    hkl = o.flip( g ) 
    # print g,"\nFlips to give\n",hkl

    assert allclose(hkl, 
                    rc_array([[1.,0.,0.],
                              [1.,1.,0.],
                              [1.,0.,1.],
                              [0.,0.,1.]],
                             direction='col').T ), str(hkl)
    assert hkl.direction == 'col'


    # print '3 long'
    """ 3 longer vectors """
    o = lattice( array( [ 10,  0,  0] , float) ,
                 array( [  0, 10,  0] , float) ,
                 array( [  0,  0, 10] , float) , direction='col' )
    assert allclose(o.r2c[:,0], array( [ 10, 0, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,1], array( [ 0, 10, 0] , float) ), str(o.r2c)
    assert allclose(o.r2c[:,2], array( [ 0, 0, 10] , float) ), str(o.r2c)
    hkl = o.flip( rc_array([10.,10.,10.], direction='col') )
    assert allclose(hkl,
                    rc_array([1.,1.,1.], direction='row') ), str(hkl)

    
    # FIXME - check is transpose of row is OK or not 
    # this break the row / col symmetry?
    
    # TODO - think on whether h / g are really col pairs
    #        Find the Sands book.



    # print hkl
    
    """ 3 really hard vectors """
    global DEBUG
    DEBUG = True
    o = lattice( array( [ 991, 990, 990] , float) ,
                 array( [ 990, 991, 990] , float) ,
                 array( [ 990, 990, 991] , float) ,direction='row')
    # Eg, these were long g-vectors far from origin but making a basis
    # Anticipate them making a ubi matrix with 990
    #assert allclose( o.c2r , array( [[ 991. ,   1. ,   1.],
    #                                 [ 990. ,   0. ,  -1.],
    #                                 [ 990. ,  -1. ,   0.],] )), str(o.c2r)
    # OK
    u = rc_array([990.,990.,990.], direction='row') 
    assert (o.nearest( u ) == rc_array([991.,990.,990], 
                                       direction='row')).all(),\
        str(o.nearest( u ))+" "+str( u )


    o = o.withvec( u ) # Remove long
    # print o.c2r
    # print o.r2c
    assert allclose(o.c2r[:,0], array( [ 1, 0, 0] , float) ), o.c2r
    assert allclose(o.c2r[:,1], array( [ 0, 1, 0] , float) ), o.c2r
    assert allclose(o.c2r[:,2], array( [ 0, 0, 1] , float) ), o.c2r
    DEBUG = False

        

if __name__=="__main__":
    import time
    start = time.time()
    test1()
    test2()
    test_eu()
    test_eu_find()
    test_fft()

    print time.time()-start
    



#!/usr/bin/env python

import sys


# ImageD11_v0.4 Software for beamline ID11
# Copyright (C) 2005  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import gzip, bz2


def getheader(filename):
    """
    Reads a header from an edf file in 1024 byte chunks.
    Assumes enclosing { }
    Returns string enclosed
    Adds a filename key at the top
    """
    h = "filename = "
    if filename[-3:]==".gz":
        fp=gzip.GzipFile(filename,"rb")
    elif filename [-4:]==".bz2":
        fp=bz2.BZ2File(filename,"rb")
    else:
        try:
            fp=open(filename,"rb")
        except IOError:
            return ""
    h=h+filename+";\n"
    s=fp.read(1024).decode()
    if s.find("{")==-1:
        raise Exception("Not an edf file")
    while 1:
        if s.find("}")>=0:
            h=h+s[0:s.find("}")+2]
            break
        else:
            h=h+s
        s=fp.read(1024).decode()
    return h

def mne(hd):
    h = {}
    order = []
    for line in hd.split(";"):
        try:
            key,vals = line.split("=")
        except ValueError:
            continue
        key = key.lstrip().rstrip()
        h[key] = vals.split(";")[0]
        order.append( key )
    for k in order:
        if k.endswith("_mne"):
            stem = k.split("_")[0]
            p = k.replace("_mne","_pos")
            newkeys = h[k].split()
            newvals = h[p].split()
            for ik, iv in zip(newkeys, newvals):
                kk = stem+":"+ik
                h[kk]=iv
                order.append( kk )
    return h, order

def main():
    keys=[]
    args = []
    for arg in sys.argv[1:]:
        if arg.find("key=")>=0:
            keys.append(arg.split("=")[-1])
            continue
        if arg.find("*")>-1 or arg.find("?")>-1:
            import glob
            args += glob.glob(arg)
        else:
            args.append(arg)
    # Print titles:
    if len(keys)>0:
        sys.stdout.write("# Filename ")
        for key in keys:
            sys.stdout.write(" %s "%(key))
        sys.stdout.write("\n")
    for arg in args:
        hd = getheader(arg)
        hd, order = mne( hd )
        sys.stdout.write(arg+" ")
        if len(keys)>0:
            for key in keys:
                if hd.has_key(key):
                    sys.stdout.write(" "+hd[key]+" ")
        else:
            for k in order:
                sys.stdout.write("%s = %s ;\n"%(k, hd[k]))
        sys.stdout.write("\n")


if __name__=="__main__":
    main()



from __future__ import print_function, division
import collections, json

"""
Load and save a data analysis project

   Images part - all the frames in one long table
               - which frames are adjacent
   We use a namedtuple for each scan point and record all the things
   that we care about as a tuple: 
          (filename, Omega, dty, z, Load, etc) <- can be hashed
   The edges tell us which frames want/need merging.

   Processing parts:
   foreach node:
       Corrected images (dark, flat, ... )
       Background image(s)
       Threshold estimation
       Label images
       List of peaks

"""


RotationNode = collections.namedtuple( "RotationNode",
                                       "filename Omega" )

DiffTomoNode = collections.namedtuple( "DiffTomoNode",
                                       "filename Omega dty" )

class Scan( object ):
    """ Scan data as a graph
    nodes = list of datapoints 
    edges = which nodes are adjacent to this node
    """
    def __init__(self,
                 nodes = None,    # type: List[]
                 edges = None,    # type: List[[int,]]
                 ):      
        """
        nodes = measurement points in the scan
        edges = edges[i] = connections to point i
        hidden self.node_index that gives you position in scan
        """
        if nodes is None:
            self.nodes = []
        else:
            self.nodes = nodes
        if edges is None:
            self.edges = []
        else:
            self.edges = edges
        # Given a node, node_index finds it in the list
        self.node_index = {}
        for i, node in enumerate(self.nodes):
            self.node_index[node] = i
        if __debug__:
            for i, e in enumerate(self.edges): # i is implicit
                for j in e:
                    assert j>=0 and i<len(nodes)

    def addNode( self, node, neighbors = [] ):
        """
        Adding another node
         node = node to be added
         neighbors = images to link up to
                     usually a list of negative integers
                     e.g. [ -1,] = previous in 1D
                          [ -1, -180] = previous in omega, dty
                     Can be empty
                     Can be for interlaced, etc
        Always stored as positive
        """
        adr  = len(self.nodes)          # adr = address in array
        self.nodes.append( node )
        self.node_index[node] = adr
        nbs = []                        # nbs = neighbors
        err = 0                         # bad refs given
        for inode in neighbors:
            if inode > 0 and inode < adr:
                neighbor = inode
            elif inode < 0 and inode >= -adr:
                neighbor = inode + adr
            else:
                err +=1 # Error
                continue
            self.edges[ neighbor ].append( adr )
            nbs.append( neighbor )
        self.edges.append( nbs )
        assert len(self.edges) == len(self.nodes)
        return err

    def neighbors( self, node ):
        """
        node = scan point (Image, Omega, etc)
        node = integer
        """
        if isinstance( i, int ):
            return self.edges[i]
        if isinstance( i, tuple): 
            i = self.node_index[ node ]
            return self.edges[i]
        raise TypeError("node not understood, want tuple or int row index")

    def __len__(self):
        return len(self.nodes)

    def __getitem__(self, i):
        """ We index to the node, not the edges ? """
        return self.nodes[i]

    def todict( self ):
        """
        Convert to a dictionary representation for serialisation
        """
        data = { "titles": self.nodes[0]._fields,
                 "rows" : [[field for field in node ]
                                 for node in self.nodes],
                 "edges" : self.edges } 
        return data 

    def fromdict( self, dct ):
        """
        Convert from a dictionary representation for serialisation
        """
        self.edges = dct['edges']
        # named tuple
        tup = collections.namedtuple( "Node", dct['titles'] )
        for i, item in enumerate( dct['rows'] ):
            t = tup( *item ) 
            self.nodes.append( t )
            self.node_index[t] = i
    

def add_motor( inputscan, motorname, motorpos ):
    """
    Adds another motor to a scan
       e.g. dty position when not in header
    """
    fields = inputscan[0]._fields + (motorname,)
    tup = collections.namedtuple( "Node", fields )
    try:
        if len( inputscan ) == len( motorpos ):
            nodes = [ tup(*(vals + (mot,))) for vals, mot in
                      zip(inputscan, motorpos)]
        else:
            raise Exception("Motorpos has wrong length")
    except TypeError: # no len
        mot = float( motorpos )
        nodes = [ tup( *(vals + (mot,))) for vals in inputscan ]
    edges = [e for e in inputscan.edges]
    return Scan( nodes, edges )


def fablescandata_from_spt( spt, headeritems=["Omega",],
                            nodeclass = RotationNode ):
    """
    Read an ImageD11 peaksearch spt file to return a Scan object
    """
    myscan = Scan()
    # Unpack the [motor|counter]_mne/_pos pairs
    motor_mne = counter_mne = []
    with open(spt, "r") as sptfile:
        for line in sptfile.readlines():
            if line[0] != "#": # skip blank and peaks
                continue
            #             0123456
            if line.find("# File ")==0:
                filename = line[7:-1] # trim newline
                header = {}
                continue
            if line.find("# motor_mne") == 0:
                motor_mne = line.split("=")[1].split()
                continue
            if line.find("# motor_pos") == 0:
                motor_pos = [float(v) for v in line.split("=")[1].split()]
                for k,v in zip( motor_mne, motor_pos ):
                    header[ k ] = v
                continue
            if line.find("# counter_mne") == 0:
                counter_mne = line.split("=")[1].split()
                continue
            if line.find("# counter_pos") == 0:
                counter_pos = [float(v) for v in line.split("=")[1].split()]
                for k,v in zip( counter_mne, counter_pos ):
                    header[ k ] = v
                continue
            if line.find("# npks") == 0:
                args = { key:float(header[key]) for key in headeritems }
                mynode = nodeclass( filename=filename, **args )
                myscan.addNode( mynode, (-1,) )
                continue
            for hitem in headeritems:
                if line.find(hitem) > 0:
                    val = line[1:].split( "=" )[-1].strip()
                    header[ hitem ] = val
            # end of frame marker
    return myscan


def sinogram_from_spt_list( sptlist ):
    scans = [fablescandata_from_spt( sptfile ) for sptfile in sptlist ]


def mergeScans( scan1, scan2 ):
    """
    Merge two scans together
    """
    n1 = len(scan1)
    n2 = len(scan2)
    # "+" joins lists:
    nodes = scan1.nodes + scan2.nodes
    edges = scan1.edges + [ (i + n1, j + n1) for i,j in edges ]
    return Scan( nodes, edge )
    

class Project( object ):
    """
    Holds all the information about the project

       scans / experiment data
       processing steps and results

    Mostly links to external files ?
    """
    def __init__(self):
        self.Scans = []
        self.processing = []
        
    def load(self):
        pass
    
    def save(self):
        pass





import numpy as np

def heightsquare(r,theta):
    """ Height of a projected square versus x """
    p = ((-1,-1,1,1),(1,-1,-1,1))
    s = np.sin(np.radians(theta))
    c = np.cos(radians(theta))
    m = ((c, -s),(s, c))
    rp = np.dot(m,p)/2
    order = np.argsort(rp[0])
    o = []
    xp = rp[0][order]
    yp = rp[1][order]
    for v in x:
        if v < xp[0]:
            o.append(0)
            continue
        if v > xp[-1]:
            o.append(0)
            continue
        if v < xp[1]:
            h1 = abs(yp[1]-yp[0])*(v-xp[0])/(xp[1]-xp[0])
            h2 = abs(yp[2]-yp[0])*(v-xp[0])/(xp[2]-xp[0])
            o.append( h1+h2 )
            continue
        if v > xp[2]:
            h1 = abs(yp[3]-yp[2])*(xp[3]-v)/(xp[3]-xp[2])
            h2 = abs(yp[3]-yp[1])*(xp[3]-v)/(xp[3]-xp[1])
            o.append( h1+h2 )
            continue
        h1 = abs(yp[1]-yp[0])
        h2 = abs(yp[2]-yp[0])*(xp[1]-xp[0])/(xp[2]-xp[0])
        o.append( h1 + h2 )
    return o

if __name__=="__main__":
    x=np.linspace(-1,1,2000)
    from pylab import *
    h = heightsquare
    plot(x,h(x,0),"-")
    plot(x,h(x,1),"-")
    plot(x,h(x,10),"-")
    plot(x,h(x,50),"-")
    plot(x,h(x,45),"-")
    plot(x,h(x,99),"-")
    show()




from __future__ import print_function


"""
Try to compute the radius/arc images required for the
fast radial regrouping method proposed by Peter Boesecke

We wish to have the x/y displacement images for the data
image to put it into an output array as if it were a spatial
distortion

(Uses the approach of the rsv_mapper method which does 3D
regrouping)

Jon Wright 18/11/2009
"""

from ImageD11 import parameters, blobcorrector, transform
from fabio import edfimage

import numpy, math


class xydisp:
    required_pars = ["wavelength", "distance", # etc
                     ]
                     
                     
    def __init__(self, splinefile = None,
                 parfile = None):
        """
        splinefile = fit2d spline file, or None for images
        that are already corrected

        parfile = ImageD11 parameter file. Can be fitted
        using old ImageD11_gui.py or newer fable.transform
        plugin.
        """
        self.splinefile = splinefile
        
        if self.splinefile is None:
            self.spatial = blobcorrector.perfect()
        else:
            self.spatial = blobcorrector.correctorclass( splinefile )

        self.parfile = parfile
        self.pars = parameters.parameters()
        self.pars.loadparameters( parfile )
        for key in self.required_pars:
            
            if key not in self.pars.parameters:
                raise Exception("Missing parameter "+str(par))

    def compute_tth_eta(self, dims):
        """
        Find the twotheta and azimuth images
        """
        assert len(dims) == 2
        xim, yim = self.spatial.make_pixel_lut( dims )
        self.dims = dims
        peaks =  [ numpy.ravel(xim), numpy.ravel(yim) ]
        tth, eta =  transform.compute_tth_eta( peaks, 
                                               **self.pars.get_parameters() )
        assert len(tth) == dims[0]*dims[1]
        assert len(eta) == dims[0]*dims[1]

        # Now we have the twotheta and azimuth images in memory
        # they are in degrees
        
        self.tth = numpy.reshape( tth, dims )
#        self.eta = numpy.mod(numpy.reshape( eta, dims ), 360)-180
        self.eta = numpy.reshape( eta, dims )-eta.mean()
        self.compute_rad_arc()
        
    def compute_rad_arc(self):
        """
        This part needs more work - how to properly define the output
        after spd is patched
        For now we aim to make a triangle filling the same array size

        # FIXME - this remains unclear to me, what are the output
        units supposed to be??
        """
        tth_rad = self.tth * math.pi / 180.0
        eta_rad = self.eta * math.pi / 180.0
        arclength = tth_rad * eta_rad

        # x-axis, eg [0], is tth
        tthmax =  numpy.max( self.tth )
        tthmin =  numpy.min( self.tth )
        tthstep = (tthmax - tthmin)/(self.dims[0] - 1)
        self.tthbin = numpy.floor( (self.tth - tthmin)/tthstep )
        self.tthvals = numpy.arange(tthmin,tthmax+tthstep*0.5,tthstep)
        # Ideally we want the arc bins to vary with tth?
        #arcmax = numpy.max( arclength )
        #arcmin = numpy.min( arclength )
        # 4 corners of image
        arcmin = arclength.min()
        arcmax = arclength.max()
        #from matplotlib.pylab import imshow,show, colorbar
        #imshow(arclength)
        #colorbar()
        #show()
        arcstep = (arcmax - arcmin)/(self.dims[1] - 1)
        arcmid  = 0.5*(arcmax+arcmin)
        # Make integer pixel id images

        self.arcbin = numpy.floor((arclength - arcmid)/arcstep )+self.dims[1]/2

        assert self.tthbin.min() >= 0
        assert self.tthbin.max() < self.dims[0], self.tthbin.max()
        assert self.arcbin.min() >= 0, self.arcbin.min()
        assert self.arcbin.max() < self.dims[1]

        # Now convert these to displacements compared to input image
        # Use the same code as for the spline case to get the "x/y" images
        ideal = blobcorrector.perfect()
        idealx, idealy = ideal.make_pixel_lut( self.dims )
        self.dx = self.tthbin - idealx
        self.dy = self.arcbin - idealy

    def write(self, stemname):
        """
        save the dx, dy images
        """
        im = edfimage.edfimage()
        im.data = self.dx
        im.write( "%s_dx.edf"%(stemname), force_type = numpy.float32)
        im = edfimage.edfimage()
        im.data = self.dy
        im.write( "%s_dy.edf"%(stemname), force_type = numpy.float32)
        numpy.save("%s_tth.npy"%(stemname),self.tthvals)

def get_options(parser):

    parser.add_option("-p", "--pars", action="store",type="string",
                      dest = "pars", default = None,
                      help = "ImageD11 parameter file for experiment")
    
    parser.add_option("-o", "--output", action="store", type="string",
                      dest = "output", default = None,
                      help = "stem name for output x/y edf images")

    parser.add_option("-s", "--splinefile", action="store", type="string",
                      dest = "spline", default = None,
                      help = "Name of fit2d spline file for spatial dist")

    parser.add_option("--nf", action="store", type="int",
                      dest = "nf", default = 2048,
                      help = "Number of pixels in fast direction, eg 2048")
    
    parser.add_option("--ns", action="store", type="int",
                      dest = "ns", default = 2048,
                      help = "Number of pixels in slow direction, eg 2048")
    
    return parser


def main():
    """
    A CLI interface
    """
    import sys, time, os, logging
    start = time.time()
    root = logging.getLogger('')
    root.setLevel( logging.WARNING )
    try:
        from optparse import OptionParser
        parser = OptionParser()
        parser = get_options( parser )
        options, args = parser.parse_args()
    except SystemExit:
        raise
    except:
        parser.print_help()
        print("\nSorry, there was a problem interpreting your command line")
        raise

        
    if options.pars is None:
        print("Failed: You must supply a parameters file, -p option")
        sys.exit()
    if not os.path.exists( options.pars ):
        print("Cannot find your file",options.pars)
        sys.exit()

    worker = xydisp(
        splinefile = options.spline,
        parfile = options.pars
        )
    
    worker.compute_tth_eta( (options.nf, options.ns) )
    worker.write( options.output )
        

        
if __name__ == "__main__":
    main()



import glob, sys

import numpy as np

from ImageD11 import cImageD11, indexing, transformer


# open up a set of ubi files

parfilename = sys.argv[1]

ubifl = glob.glob(sys.argv[2])

#for a in glob.glob(sys.argv[3:]):
#   ubifl += glob.glob(a)

ubi_all = {}
for fname in ubifl:
    print fname
    ubi_all[fname] = indexing.readubis(fname)
print "OK"




# Now try to make these into a unique set.
#
# For each ubi:
#     compute the reciprocal peak positions of central lattice
#     see if these match a previous one.

pks = [
        ( 0, 0, 1), ( 0, 1, 0), ( 1, 0, 0),
        ( 0, 0,-1), ( 0,-1, 0), (-1, 0, 0),
        ( 0, 1, 1), ( 1, 1, 0), ( 1, 0, 1),
        ( 0,-1, 1), ( 1,-1, 0), (-1, 0, 1),
        ( 0, 1,-1), (-1, 1, 0), ( 1, 0,-1),
        ( 0,-1,-1), (-1,-1, 0), (-1, 0,-1),
        ( 1, 1, 1), (-1, 1, 1), ( 1,-1, 1),
        ( 1, 1,-1), (-1,-1, 1), (-1, 1,-1),
        ( 1,-1,-1), (-1,-1,-1)
        ]

hkl = np.array(pks,np.float).T

uniq_ubis = []
names = ubi_all.keys()
#dsu = [ (int( np.split(".")[0].split("_")[-1] ), n) for n in names ]
#dsu.sort()
#names = [d[1] for d in dsu]

tol = 0.05
for name in names[15:]:
    this_list = ubi_all[name]
    for ubi, i in zip(this_list,range(len(this_list))):
        gv = np.dot(np.linal.inv(ubi),hkl)
        seen = 0
        for j in range(len(uniq_ubis)):
            u = uniq_ubis[j][0]
            npk=cImageD11.score(u,gv.T,tol)
            if npk == len(pks):
                # seen you before
                uniq_ubis[j][1].append((name, i))
                seen += 1
            if npk > 12 and npk < len(pks):
                print "close...",npk,
        if seen == 0:
            uniq_ubis.append([ubi,[(name,i)]])
        if seen > 1:
            print "uniq seen more than once",ubi,i,name

print "Found",len(uniq_ubis),"unique ubi matrices"


z={}
for line in open("job_z.txt","r").readlines():
    j,zh = line.split()

    z [ int(j)-1 ] = float(zh)

def get_z(name):
    j=int(name.split(".")[0].split("_")[-1])
    return j,z[j]


for entry in uniq_ubis:
    print "\n\n"
    for i in range(3):
        for j in range(3):
            print "# ubi[%d,%d] = %f"%(i,j,ubi[i,j])
    ubi = entry[0]

    for (name,i) in entry[1]:
        j,zh = get_z(name)
        print j,"%7.5f"%(zh),name,i,

        # we have the ubi matrix here.
        # we want to refine and score this ubi against some data
        try:
            t = transformer.transformer()
            t.loadfileparameters(parfilename)
            t.loadfiltered(name.replace(".ubi",""))
            t.computegv()
        except:
            print name
            raise

        # bit of a hack - between 10 and 11 degrees
        h = np.matrixmultiply(ubi, t.gv)
        hint = np.floor(h + 0.5).astype(np.int) # rounds down
        diff = h - hint
        drlv2 = np.sum(diff * diff,0)
        ind = np.compress( drlv2 < tol*tol, np.arange(t.twotheta.shape[0]))
        avg = 4
        npix = 3
        intensity = np.sum((t.finalpeaks[avg,:]*t.finalpeaks[avg,:])[ind])
        print cImageD11.score(entry[0],t.gv.T,tol), intensity


# now re




from __future__ import print_function
# ImageD11_v0.4 Software for beamline ID11
# Copyright (C) 2005  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""
Some routines for factor analysis, a bit slow, via the SVD
"""
import glob, struct, logging

import numpy as np

from ImageD11 import opendata

class factors:
    """
    Class for factor analysis
    """
    def __init__(self):
        self.obsdata = None
        self.svd = None
        self.gendata = None
        self.x = None
        self.nfactors=0

    def generatedata(self):
        """
        compute the data from our svd
        """
        if self.svd is not None:
            l,s,r=self.svd # left , singularvals, right
            nf=self.nfactors
            I=np.identity(nf,np.float)
            suse=I*s[:nf]
            ls = np.dot(l[:,:nf], suse)
            self.gendata=np.dot(ls,r[:nf,:])
            logging.debug("generated data.shape"+str(self.gendata.shape))


    def factorsvd(self):
        """
        compute the svd
        """
        logging.debug("In svd")
        if self.obsdata is not None:
            logging.debug("Calling svd")
            self.svd = np.linalg.svd(self.obsdata)

    def savesvd(self,filename):
        """
        Save the time consuming svd step
        """
        if self.svd is not None:
            l,s,r=self.svd
            out=open(filename,"wb")
            out.write(struct.pack("lllll",
                                  l.shape[0],
                                  l.shape[1],
                                  s.shape[0],
                                  r.shape[0],
                                  r.shape[1]))
            print(len(l.astype(np.float).tostring()))
            out.write(l.astype(np.float).tostring())
            out.write(s.astype(np.float).tostring())
            out.write(r.astype(np.float).tostring())
            out.close()

    def loadsvd(self,filename):
        """
        Save the time consuming svd step
        """
        out=open(filename,"rb")
        dims=struct.unpack("lllll",out.read(struct.calcsize("lllll")))
        print(dims,8*dims[0]*8*dims[1])
        l=np.fromstring(out.read(8*dims[0]*dims[1]),np.float)
        s=np.fromstring(out.read(8*dims[2]),np.float)
        r=np.fromstring(out.read(8*dims[3]*dims[4]),np.float)
        print(l.shape,s.shape,r.shape,dims)
        l=np.reshape(l,(dims[0],dims[1]))
        r=np.reshape(r,(dims[3],dims[4]))
        self.svd=(l,s,r)



    def setnfactors(self,n):
        """Decide on the number of factors in the data"""
        self.nfactors=n
        print("Number of factors set to",self.nfactors)

    def loadchis(self,filename):
        """
        Glob for filenames
        """
        fl=glob.glob(filename[:-8]+"????"+".chi")
        fl.sort()
        print("Number of chi files is:",len(fl))
        dl=[opendata.openchi(f).data[:,1] for f in fl]
        self.obsdata=np.array(dl)
        self.x=opendata.openchi(fl[0]).data[:,0]
        print(self.x.shape,self.obsdata.shape)

    def saveobsdata(self,filename):
        """
        Save in binary format?
        """
        out=open(filename,"wb")
        out.write(struct.pack("lll",self.x.shape[0],
                                    self.obsdata.shape[0],
                                    self.obsdata.shape[1]))
        out.write(self.x.astype(np.float).tostring())
        out.write(self.obsdata.astype(np.float).tostring())
        out.close()

    def readobsdata(self,filename):
        """ Reads observed data from a binary format """
        infile=open(filename,"rb")
        sizes=struct.unpack("lll",infile.read(struct.calcsize("lll")))
        # Type is Float therefore 8 bytes per item
        print(sizes)
        self.x=np.fromstring(infile.read(sizes[0]*8),np.float)
        self.obsdata=np.fromstring(infile.read(8*sizes[1]*sizes[2]),np.float)
        print(self.obsdata.shape)
        self.obsdata=np.reshape(self.obsdata,(sizes[1],sizes[2]))
        print(self.x.shape,self.obsdata.shape)


if __name__=="__main__":
    o=factors()
    import sys
    if sys.argv[1].find("chi")>-1:
        o.loadchis(sys.argv[1])
    else:
        o.readobsdata(sys.argv[1])
    print(dir(o))
    o.setnfactors(int(sys.argv[2]))
    o.factorsvd()
    o.generatedata()




import unittest
import numpy as np
import scipy.sparse
from ImageD11 import cImageD11

def _sum_duplicates(row, col, data):
    """ for testing - to be moved or removed """
    # From scipy.sparse.coo_matrix
    # Assumes (data, row, col) not in canonical format.
    if len(data) == 0:
        return row, col, data
    order = np.lexsort((col,row))
    row = row[order]
    col = col[order]
    data = data[order]
    unique_mask = ((row[1:] != row[:-1]) |
                   (col[1:] != col[:-1]))
    unique_mask = np.append(True, unique_mask)
    row = row[unique_mask]
    col = col[unique_mask]
    unique_inds, = np.nonzero(unique_mask)
    data = np.add.reduceat(data, unique_inds, dtype=data.dtype)
    return row, col, data            


class testpls(unittest.TestCase):
    N=1000
    def test1(self):
        np.random.seed(42)
        N = self.N
        a = np.random.random(N*2)
        i, j = (a*10).astype('i').reshape(2,N)
        oj = np.empty( N, 'i')
        oi = np.empty( N, 'i')
        tmp = np.empty( 20, 'i' )
        d = np.ones( N, np.int )
        r,c,d = _sum_duplicates( i, j, d )
        n = cImageD11.compress_duplicates( i, j, oi, oj, tmp )
        self.assertTrue( n == len(r) )
        self.assertTrue((r==i[:n]).all())
        self.assertTrue((c==j[:n]).all())
        self.assertTrue((d==oi[:n]).all())
    def test2(self):
        np.random.seed(42)
        N = self.N
        a = np.random.random(N*2)
        i, j = (a*10).astype('i').reshape(2,N)
        oj = np.empty( N, 'i')
        oi = np.empty( N, 'i')
        tmp = np.empty( 20, 'i' )
        d = np.ones( N, np.int )
        coo = scipy.sparse.coo_matrix((d,(i.copy(),j.copy())))
        csr = coo.tocsr()
        coo = csr.tocoo()
        #print("",i,"\n",j)
        n = cImageD11.compress_duplicates( i, j, oi, oj, tmp )
        self.assertEquals(cImageD11.sparse_is_sorted( i[:n].astype(np.uint16),
                                                      j[:n].astype(np.uint16)
                                                  ), 0)
        r = coo.row
        c = coo.col
        d = coo.data
        self.assertTrue( n == len(r) )
        self.assertTrue((r==i[:n]).all())
        self.assertTrue((c==j[:n]).all())
        self.assertTrue((d==oi[:n]).all())

    
if __name__=="__main__":
    unittest.main()
            




from __future__ import print_function

from ImageD11.columnfile import columnfile
from ImageD11.parameters import read_par_file
from ImageD11.unitcell   import unitcell_from_parameters
from ImageD11.grain      import read_grain_file, write_grain_file
from ImageD11 import transform
import numpy as np

# idea:
#    Rotate XL,YL,ZL into laboratory frame
#    Rotate beam into laboratory frame
#    Make a new compute-g-vectors where rotations were already applied
#    Fit grain position and orientation in this frame


def getCxyz(colf, pars):
    """
    Find spot positions in crystal frame
    colf = columnfile (or other object) holding .sc, .fc, .omega
    pars = parameters for ImageD11 transform module
    """
    wedge = pars.get("wedge")
    chi   = pars.get("chi")
    peaks_xyz = transform.compute_xyz_lab([ colf.sc,
                                            colf.fc ],
                                          **pars.parameters)
    fun = transform.compute_g_from_k
    peaks_Cxyz = fun( peaks_xyz, colf.omega, 
                      wedge=wedge, 
                      chi=chi)
    b = np.zeros(peaks_Cxyz.shape)
    b[0,:] = 1.0/pars.get("wavelength")
    beam_Cxyz =  fun( b, colf.omega, 
                      wedge=wedge, 
                      chi=chi)
    return peaks_Cxyz, beam_Cxyz

def compute_Cgve(txyz, peaks_Cxyz, beam_Cxyz, wavelength):
    """
    Computes g-vectors from spots and beam in crystal frame
    txyz = (3,) origin position
    peaks_Cxyz = (3,n) spot position on rotated detector
    beam_Cxyz  = (3,n) vectors of rotated incident beam
                       beam is already normalised to wavelength
    wavelength = float radation, normalises length
    returns scattering vectors (g-vectors)
    """
    # Output ray - full length
    r_f = (peaks_Cxyz.T - txyz).T
    # Normalise to unit vector and scale for wavelength
    fac =  wavelength * np.sqrt( (r_f*r_f).sum(axis=0) )
    np.divide( r_f, fac, r_f )
    # Compute scattering vector
    np.subtract( r_f, beam_Cxyz, r_f )
    return r_f

def compute_dgdt( txyz, peaks_Cxyz, beam_Cxyz, wavelength):
    """
    Compute the gvectors and the derivatives with respect to tx,ty,tz
    txyz = (3,) origin position
    peaks_Cxyz = (3,n) spot position on rotated detector
    beam_Cxyz  = (3,n) vectors of rotated incident beam
                       beam is already normalised to wavelength
    wavelength = float radation, normalises length
    returns scattering vectors (g-vectors) and d(g)/d(txyz)
    """
    r = (peaks_Cxyz.T - txyz).T # 3xn
    # Normalise to unit vector and scale for wavelength
    f = wavelength * np.sqrt( (r*r).sum(axis=0) )
    # d|r| -> r/|r| and f/wvln = |r|
    dfdT = - wavelength * wavelength * r / f # 3xn
    # over-writes in place
    np.divide( r, f, r )
    # quotient rule d(r/f) : (r'.f - r.f')/f.f
    drdT = np.zeros((3,3,len(f)))
    # d( r/f )/dT -> r is a vector, r[0], r[1], r[2]
    #                f is a scalar
    # o = np.ones(len(f))
    # please derive/explain this... note r is already r/f
    drdT[0] = ( (dfdT[0] * r).T + (1,0,0) ).T / f
    drdT[1] = ( (dfdT[1] * r).T + (0,1,0) ).T / f
    drdT[2] = ( (dfdT[2] * r).T + (0,0,1) ).T / f
    np.subtract( r, beam_Cxyz, r )
    return r, drdT


def fit_ub_t( ub, translation, hkl, peaks_Cxyz, beam_Cxyz, wavelength):
    """
    Fits the ub and grain origin to a list of assigned peaks
    All unit cell and orientations parameters are free
    Runs 2 cycles (empirically this converges)
    
    ub = (3,3) input UB matrix (so that h ~= UB.g)
    translation = (3,) grain origin
    hkl = (3,n) integer peak assignments
    peaks_Cxyz = (3,n) spot positions in crystal frame (getCxyz)
    beam_Cxyz = (3,n) beam directions in crystal frame (getCxyz)
    wavelength = float, radiation, normalises length
    returns fitted UB and translation
    """
    npk = len(hkl[0])
    dg = np.zeros( (12,3,npk) )
    for i in range(3):
        for j in range(3):
            dg[i*3+j,i] = hkl[j] # ub in 0->8
            # Why? :
            # d(gcalc3)/d(ub9) = h
            # dgdub = np.zeros((3,3,3,c.nrows))
            # g0 = ub00.h0 + ub01.h1 + ub02.h2
            # g1 = ub10.h0 + ub11.h1 + ub12.h2
            # g2 = ub20.h0 + ub21.h1 + ub22
            # only 9 out of 27 non-zero. Should roll this another way. (outer h,h.T)
            # ubi ubj gk
            # print dgdub[0,0,0].shape, hi[0].shape
            # dgdub[0,0,0]=hi[0]
            # dgdub[0,1,0]=hi[1]
            # dgdub[0,2,0]=hi[2]
            # dgdub[1,0,1]=hi[0]
            # dgdub[1,1,1]=hi[1]
            # dgdub[1,2,1]=hi[2]
            # dgdub[2,0,2]=hi[0]
            # dgdub[2,1,2]=hi[1]
            # dgdub[2,2,2]=hi[2]
            # gobs = g0 + dgdT.t
            # gobs - gcalc = dgdT.t
            # t = (gobs - gcalc).
    tnew = translation.copy()
    ubnew = ub.copy()
    # empirically it converges to 3 decimal places in 1 cycle
    # ...since: dgobsdt seems to depend on t we run a couple of cycles
    for _ in range(2):
        gobs, dgobsdt = compute_dgdt( tnew, peaks_Cxyz, beam_Cxyz, wavelength )
        # Note dgdub=h does not change here
        for i in range(3):
            dg[i+9] = dgobsdt[i]       # translation at 9,10,11
        gcalc =  np.dot( ubnew , hkl ) # 3xn
        gdiff = gcalc - gobs
        #    print((gdiff*gdiff).ravel().sum(),tnew)
        dg.shape = 12,3*npk
        mat = np.dot( dg, dg.T )
        rhs = np.dot( dg, gdiff.ravel() )
        imat = np.linalg.inv( mat )
        shifts = np.dot (imat, rhs )
        dg.shape =  (12,3,npk) 
        ubnew = ubnew  - np.reshape(shifts[:9],(3,3))
        tnew  = tnew - shifts[9:]
    return ubnew, tnew

def fitagrain( gr, pars ):
    """
    """
    t = gr.translation.copy()
    ub = gr.ub.copy()
    pks, beam = getCxyz( gr, pars )
    # This is a little ugly. Can the wavelength live in the B matrix?
    hi = np.round( np.dot( gr.ubi, compute_Cgve( t, pks, beam, pars.get("wavelength") ) ) )
    ubnew, tnew = fit_ub_t( ub, t, hi, pks, beam, pars.get("wavelength"))
    return ubnew, tnew


def main():
    import sys, time
    c = columnfile(sys.argv[1])
    p = read_par_file(sys.argv[2])
    u = unitcell_from_parameters(p)
    gl = read_grain_file(sys.argv[3])
    if gl[0].translation is None:
        gl[0].translation = np.array((0.,0.,0.))
    start = time.time()
    # Setup and assign hkls
    w = p.get("wavelength")
    peaks_Cxyz, beam_Cxyz = getCxyz( c, p )
    t = gl[0].translation.copy()
    ub = gl[0].ub.copy()
    ubi = gl[0].ubi.copy()
    gve = compute_Cgve(t, peaks_Cxyz, beam_Cxyz, w)
    hi = np.round( np.dot( ubi, gve ) )
    lastgof = 1e9
    ubn, tn = fit_ub_t( ub, t, hi, peaks_Cxyz, beam_Cxyz, w)
    print("Before\nt=",gl[0].translation)
    print("UB=",gl[0].ub)
    gl[0].set_ubi( np.linalg.inv( ubn ) )
    gl[0].translation = tn    
    dt = time.time() - start
    print("time calculating",dt,"gps",1/dt)
    print("After\nt=",gl[0].translation)
    print("UB=",gl[0].ub)
    write_grain_file(sys.argv[4],gl)


def main2():
    import sys
    c = columnfile(sys.argv[1])
    p = read_par_file(sys.argv[2])
    gl = read_grain_file(sys.argv[3])
    for i,g in enumerate(gl):
        mask = c.labels == i
        g.sc = np.compress( mask, c.sc )
        g.fc = np.compress( mask, c.fc )
        g.omega = np.compress( mask, c.omega )
        ubnew, tnew = fitagrain( g, p )
        g.set_ubi( np.linalg.inv( ubnew ) )
        g.translation[:] = tnew
        print(i, len(g.sc), tnew)
    write_grain_file( sys.argv[4], gl )
        




if __name__=="__main__":
    main2()






# python rotdex.py g10.flt cubic.par g10.ubi g10.t1



""" Write some testcases for the image scaling stuff """
import unittest

import numpy
from ImageD11.scale import scale

class testscale(unittest.TestCase):
    """old testcase """
    def testscaleimage(self):
        """check the object works """
        im1 = numpy.ones((10, 10), numpy.float)
        im1[2:8, 2:8] *= 2
        im1[4:6, 4:6] *= 2
        im2 = im1 * 2. + 3.
        o = scale(im1)
        a , b = o.scale(im2)
        self.assertEqual( abs(a-2.)< 1e-6 and abs(b-3.) < 1e-6 , True)
        scaled = o.scaleimage(im2)
        diff = numpy.ravel(scaled - im1)
        self.assertEqual( numpy.sum(diff*diff) < 1e-6 , True)

if __name__ == "__main__":
    unittest.main()




from __future__ import print_function


import sys, numpy, time
from ImageD11 import cImageD11, blobcorrector, ImageD11_file_series
from fabio.openimage import openimage
from fabio.edfimage import edfimage

def get_options(parser):
    """
    Command line interface for making a mapping
    Add our options to a parser object
    """
    parser = ImageD11_file_series.get_options( parser )
    parser.add_option("--lookup", action="store",type="string",
                      dest="lookup",default="None",
                      help="Lookup tables stem name")
    parser.add_option("--outdir", action="store",type="string",
                      dest="outdir",default=None,
                      help="Output directory")
    parser.add_option("--mask", action="store",type="string",
                      dest="mask",default=None,
                      help="fit2d mask file")

    return parser


def main():
    """
    A CLI user interface
    """
    import sys, time, os, logging
    start = time.time()

    root = logging.getLogger('')
    root.setLevel(logging.WARNING)

    try:
        from optparse import OptionParser
        parser = OptionParser()
        parser = get_options( parser )
        options, args = parser.parse_args()
    except SystemExit:
        raise
    except:
        parser.print_help()
        print("\nProblem with your options:")
        raise

    if options.mask is not None:
        fit2dmask = (1-openimage(options.mask).data).ravel()
    else:
        fit2dmask = 1.0

    first_image = True

    imagefiles = ImageD11_file_series.get_series_from_options( options, args )

    tthvals = numpy.load(options.lookup+"_tth.npy")

    try:
        for fim in imagefiles:
            dataim = fim.data
            print(fim.filename)
            if first_image: # allocate volume, compute k etc

                first_image = False


                dxim = openimage(options.lookup+"_dx.edf").data
                dyim = openimage(options.lookup+"_dy.edf").data

                outsum = numpy.ravel( numpy.zeros( dataim.shape,
                                                   numpy.float32 ) )
                outnp = numpy.ravel( numpy.zeros( dataim.shape,
                                                  numpy.float32 ) )

                e = edfimage()


                # C code from rsv_mapper (not intended to be obfuscated)
                o = blobcorrector.perfect( )
                idealx, idealy = o.make_pixel_lut( dataim.shape )

                destx, desty = idealx + dxim, idealy + dyim
    
                assert destx.min() >= 0
                assert destx.max() < dataim.shape[1]
                assert desty.min() >= 0
                assert desty.max() < dataim.shape[1]

                imageshape = dataim.shape
                
                indices = numpy.ravel( destx ).astype( numpy.intp )
                numpy.multiply( indices, dataim.shape[1], indices )
                numpy.add( indices,
                           numpy.ravel( desty ).astype( numpy.intp ),
                           indices )

                assert indices.min() >= 0
                assert indices.max() < dataim.shape[0]*dataim.shape[1]
                on = numpy.ones(len(outnp), numpy.float32)
                if fit2dmask is not None:
                    on = on * fit2dmask
                    
                # Number of pixels and mask are constant
                cImageD11.put_incr( outnp,
                                  indices,
                                  on )

                mask = outnp < 0.1
                scalar = ( 1.0 - mask ) / ( outnp + mask )
                    
                flatshape = outsum.shape

                # 

                arsorted = mask.copy()
                outmask = mask.copy()
                outmask = outmask * 1e6
                outmask.shape = imageshape
                arsorted.shape = imageshape
                arsorted.sort(axis=1)
                minds = numpy.array([ l.searchsorted(0.5) for l in arsorted ])

                

            # ENDIF firstimage


            
            start = time.time()

            numpy.multiply( outsum, 0, outsum )
            outsum.shape = flatshape

            dm = (dataim.ravel()*fit2dmask).astype(numpy.float32)
            
            cImageD11.put_incr( outsum,
                              indices,
                              dm )

            # outsum = outsum.reshape( dataim.shape )
            # outnp = outnp.reshape( dataim.shape ).astype(numpy.int32)
            # Normalise
            numpy.multiply( outsum, scalar, outsum )
            print(dataim.max(),dataim.min(), end=' ')
            print(scalar.max(),scalar.min(),outsum.min(), outsum.max( ))

            outsum.shape = imageshape
            # saving edf
            e.data=outsum
            e.write( "r_"+fim.filename  , force_type=numpy.float32)
            
            print(time.time()-start)

            


    except:
        raise

#f = open("ar.bin","w")
#f.write(outsum.tostring())
#f.write(outnp.tostring())


    
        
if __name__ == "__main__":
    main()





import fabio, numpy as np, h5py

# read in .txt file and .dat files

def readtxt(fname):
    """
    Reads txt file written by spec and fills in table of:
      filenumber, diffty, diffrz, pico3, current
    """
    cnext = None
    h = {}
    vals = []
    ctrs = "diffty pico3 notsrcu".split()
    h["pico3"]=1
    h["notsrcu"]=1
    for line in open(fname):
        if line[0] == "#":
            if cnext is not None:
                h[cnext] = float(line.split()[cindex])
                cnext = None
            for c in ctrs:
                if line.find(c)>0:
                    cnext=c
                    cindex=line.split().index(c)
                    break
            continue
        items = line.split()
        if len(items) == 0:
            continue
        fname = items[0]
        angle = 0.5*(float(items[4]) + float(items[5]))
        #print fname, fabio.getnum(fname), diffty, angle, pico3
        vals.append([fabio.getnum(fname), angle] + [h[c] for c in ctrs])
        # print vals[-1]
    return np.array(vals)
                
def decidegrid(vals):
    """
    Figure out a position and angle step from vals array
      number, angle, ypos
    """
    y0 = vals[0,2] # y of first image
    for i,y in enumerate(vals[:,2]):
        if y!=y0: break
    blocksize = i
    if len(vals) % blocksize != 0:
        print("Blocksize seems to be",blocksize, end=' ')
        print("you have",len(vals),"images. Problem.")
    vals.shape = vals.shape[0]/blocksize,blocksize,vals.shape[1]
    # Find the best angular grid mapping onto this
    a0=vals[0,:,1]
    a1=vals[1,:,1]
    offseterr=[abs(a0[:-i]-a1[:-i][::-1]).sum() for i in range(1,20)]
    return vals, np.argmin(offseterr)

def myloadtxt(fname, header=17):
    return np.array([float(l.split()[1]) for l in open(fname).readlines()[header:]])

def readdat(stem,vp):
    fname = "%s%04d.dat"%(stem,vp[0,0,0])
    dat = np.loadtxt( fname )
    xaxis = dat[:,0]
    fullarray = np.zeros((vp.shape[0],vp.shape[1],len(dat)), np.float32)
    n=vp.shape[0]
    # Normalise to srcur here:
    srcurmean = vp[:,:,-1].mean()
    for i in range(n):
        sys.stdout.write("%4d / %4d \r"%(i,n))
        sys.stdout.flush()
        for j in range(vp.shape[1]):
            fname="%s%04d.dat"%(stem,vp[i,j,0])
            scale = srcurmean/vp[i,j,-1]
            fullarray[i,j,:] = myloadtxt( fname ) * scale
    return xaxis, fullarray

def savearray(ar,name,grp):
    if name not in grp:
        dataset = grp.create_dataset(
            name=name,
            shape=ar.shape,
            dtype="float32")
        dataset[:] = ar[:]

def writehdf(fname, blob, tth, vp):
    hdf5path = "DiffTomo/NXdata/sinogram"
    dirname = "DiffTomo/NXdata"
    basename= "sinogram"
    h = h5py.File(fname)
    group = h.require_group(dirname)
    savearray(tth, "zaxis", group)
    savearray(vp[:,:,2].mean(axis=1), "yaxis", group)
    savearray(vp[:,:,1].mean(axis=0), "xaxis", group)
    savearray(vp, "valsarray", group)
    savearray(blob, basename, group)
    h.close()




if __name__=="__main__":
    import sys, os
    HOME = "/data/visitor/ev78/id11"
    os.chdir(HOME)
    stem = sys.argv[1]
    print(stem)
    txtfile = "%s/%s.txt"%(stem,stem)
    # file with motor positions and image filenames
    vals = readtxt(txtfile)
    # figure array dimensions
    vals, offset = decidegrid(vals)
    print(vals.shape)
    # Read 1D integrated data (pyFAI was already run on it)
    xaxis, blob = readdat( "%s/%s"%(stem,stem), vals)

    # Clipping the extra patterns due to scan problem (too many images)
    b=blob[:,:-offset].copy()
    # Throw out the last offset images (from decidegrid code)
    b[0::2]=blob[0::2,:-offset]
    # Flip every other one as we scan 0->180, 180->0
    b[1::2]=blob[1::2,:-offset][:,::-1]
    v = vals[:,:-offset].copy()
    v[0::2] = vals[0::2,:-offset]
    v[1::2] = vals[1::2,:-offset][:,::-1]
    outname = os.path.join("/data/visitor/ev78/id11/voldata",stem)+".hdf"
    writehdf(outname,  b, xaxis, v)




from __future__ import print_function



def one_item_func( grain, par ):
    diff = grains[i] - par*42. - 1.
    return diff*diff

def make_sum_func( n ):
    call = 'lambda p_'+', p_'.join([str(i) for i in range(n)]) 
    call += ' : '
    for i in range(n):
       call +=  '+ one_item_func( %d, p_%d )'%(i,i)
    return eval( call )

f = make_sum_func(3)
grains = {}
for i in range(10):
    grains[i] = 43

import minuit
m = minuit.Minuit( f )

m.migrad()
print(m.values)

f = make_sum_func(4)
grains = {}
for i in range(4):
    grains[i] = -3.

print('HELLO')
m = minuit.Minuit( f )

m.migrad()
print(m.values)




from __future__ import print_function, division

## Automatically adapted for numpy.oldnumeric Sep 06, 2007 by alter_code1.py




# ImageD11_v0.4 Software for beamline ID11
# Copyright (C) 2005  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#
#
#
import logging, math

import numpy as np 
from numpy.linalg import inv
from xfab import tools, sg




def radians(x):
    return x*math.pi/180.

def degrees(x):
    return x*180./math.pi



def cross(a,b):
    """
    a x b has length |a||b|sin(theta)
    """
    return np.array([ a[1]*b[2]-a[2]*b[1] ,
                         a[2]*b[0]-b[2]*a[0] , 
                         a[0]*b[1]-b[0]*a[1] ],np.float)



def norm2(a):
    """
    Compute the unit 2 norm
    """
    return np.sqrt(np.dot(a,a))


def unit(a):
    """
    Normalise vector a to unit length
    """
    try:
        return a/norm2(a)
    except:
        logging.error("cannot normalise to unit length a=%s"%(str(a)))
        raise


# Systematic absences

def P(h,k,l):
    return False

def A(h,k,l):
    return (k+l)%2 != 0

def B(h,k,l):
    return (h+l)%2 != 0

def C(h,k,l):
    return (h+k)%2 != 0

def I(h,k,l):
    return (h+k+l)%2 != 0

def F(h,k,l):
    return (h+k)%2!=0 or (h+l)%2!=0 or (k+l)%2!=0
                                                                                
def R(h,k,l):
    return (-h+k+l)%3 != 0


outif = {
    "P" : P ,
    "A" : I ,
    "B" : B ,
    "C" : C ,
    "I" : I ,
    "F" : F ,
    "R" : R}

def cellfromstring(s):
    items = s.split()
    # print items
    latt = [float(x) for x in items[0:6]]
    try:
        symm = items[6]
    except IndexError:
        symm = 'P'
    return unitcell(latt, symm)

class unitcell:
    # Unit cell stuff
    # Generate a list of peaks from a unit cell
    def __init__(self, lattice_parameters, symmetry = "P", verbose = 0 ):
        """
        Unit cell class
        supply a list (tuple etc) of a,b,c,alpha,beta,gamma
        optionally a symmetry, one of "P","A","B","C","I","F","R"
        """
        self.lattice_parameters = np.array(lattice_parameters)
        if self.lattice_parameters.shape[0]!=6:
            raise Exception("You must supply 6 lattice parameters\n"+\
                            "      a,b,c,alpha,beta,gamma")
        self.symmetry = symmetry
        if self.symmetry not in ["P","A","B","C","I","F","R"]:
            raise Exception("Your symmetry "+self.symmetry+\
                            " was not recognised")
        # assigning a function here!
        self.absent = outif[self.symmetry]
        a = self.lattice_parameters[0]
        b = self.lattice_parameters[1]
        c = self.lattice_parameters[2]
        self.alpha=radians(self.lattice_parameters[3])
        ca= math.cos(radians(self.lattice_parameters[3]))
        cb= math.cos(radians(self.lattice_parameters[4]))
        cg= math.cos(radians(self.lattice_parameters[5]))
        if verbose==1: print("Unit cell",self.lattice_parameters)
        self.g = np.array( [[ a*a    ,  a*b*cg, a*c*cb ],
                               [ a*b*cg ,  b*b   , b*c*ca ],
                               [ a*c*cb ,  b*c*ca, c*c    ]],np.float)
        if verbose==1: print("Metric tensor\n",self.g)
        try:
            self.gi = inv(self.g)
        except:
            raise Exception("Unit cell was degenerate, could not determine"+\
                    "reciprocal metric tensor")
        if verbose==1: print("Reciprocal Metric tensor\n",self.gi)
        self.astar=np.sqrt(self.gi[0,0])
        self.bstar=np.sqrt(self.gi[1,1])
        self.cstar=np.sqrt(self.gi[2,2])

        self.alphas=degrees(math.acos(self.gi[1,2]/self.bstar/self.cstar))
        self.betas =degrees(math.acos(self.gi[0,2]/self.astar/self.cstar))
        self.gammas=degrees(math.acos(self.gi[0,1]/self.astar/self.bstar))
        if verbose==1: print("Reciprocal cell")
        if verbose==1: 
            print(self.astar, self.bstar, self.cstar, \
                self.alphas, self.betas, self.gammas)
        # Equation 3 from Busing and Levy
        self.B = np.array ( 
            [ [ self.astar , 
                self.bstar*math.cos(radians(self.gammas)) , 
                self.cstar*math.cos(radians(self.betas)) ] ,
              [ 0 , 
                self.bstar*math.sin(radians(self.gammas)) , 
                -self.cstar*math.sin(radians(self.betas))*ca ],
              [ 0 , 0  ,
                1./c ] ] , np.float)
        if verbose == 1: print(self.B)
        if verbose == 1: 
            print(np.dot( np.transpose(self.B),
                               self.B)-self.gi) # this should be zero
        self.hkls = None
        self.peaks = None
        self.limit = 0
        
        # used for caching
        self.anglehkl_rings = None
        self.anglehkl_cache = None
        self.ringtol = 0.001        

    def tostring(self):
        """
        Write out a line containing unit cell information
        """
        return "%f %f %f %f %f %f %s"%(self.lattice_parameters[0],
              self.lattice_parameters[1],
              self.lattice_parameters[2],
              self.lattice_parameters[3],
              self.lattice_parameters[4],
              self.lattice_parameters[5],
              self.symmetry)


    def gethkls_xfab(self,dsmax,spg):
        """
        Generate hkl list
        Argument dsmax is the d* limit (eg 1/d)
        Argument spg is the space group name, e.g. 'R3-c'
        """
        stl_max = dsmax/2.
        raw_peaks = tools.genhkl_all(self.lattice_parameters, 
                                 0 , stl_max,
                                 sgname=spg,
                                 output_stl=True)
        peaks = []
        for i in range(len(raw_peaks)):
            peaks.append([raw_peaks[i,3]*2,(raw_peaks[i,0],raw_peaks[i,1],raw_peaks[i,2])])
        self.peaks = peaks
        self.limit = dsmax
        return peaks

    def gethkls(self,dsmax):
        """
        Generate hkl list
        Argument dsmax is the d* limit (eg 1/d)
        Default of zero gives only the (000) reflection

        assumes [h|k|l] < 200
        """
        if dsmax == self.limit and self.peaks is not None:
            return self.peaks
        h=k=0
        l=1 # skip 0,0,0
        hs=ks=ls=1
        b=0
        peaks=[]
        while abs(h)<200: # H
            while abs(k)<200: # K
                while abs(l)<200: #L
                    ds=self.ds([h,k,l])
                    if ds < dsmax:
                        if not self.absent(h,k,l):
                            peaks.append([ds,(h,k,l)])
                        else:
                            pass
                        b=0
                    else:
                        if ls==1:
                            ls=-1
                            l=0
                        else:
                            ls=1
                            l=0
                            b=b+1
                            break
                    l=l+ls
                k=k+ks
                # l is always zero here
                if b>1:
                    if ks==1:
                        ks=-1
                        k=-1
                    else:
                        ks=1
                        k=0
                        b=b+1
                        break
            h=h+hs
            if b>3:
                if hs==1:
                    hs=-1
                    h=-1
                else:
                    hs=1
                    h=0
                    break

        peaks.sort()
           
        self.peaks=peaks
        self.limit=dsmax
        return peaks

    def ds(self,h):
        """ computes 1/d for this hkl = hgh """
        return math.sqrt(np.dot(h,np.dot(self.gi,h))) # 1/d or d*


    def makerings(self,limit,tol=0.001):
        """
        Makes a list of computed powder rings
        The tolerance is the difference in d* to decide
        if two peaks overlap
        """
        self.peaks=self.gethkls(limit+tol) # [ ds, [hkl] ]
        self.ringds=[]   # a list of floats
        self.ringhkls={} # a dict of lists of integer hkl
        # Append first peak
        peak = self.peaks[0]
        self.ringds.append(peak[0])
        self.ringhkls[peak[0]] = [peak[1]]
        for peak in self.peaks[1:]:
            if abs(peak[0] - self.ringds[-1]) < tol:
                self.ringhkls[self.ringds[-1]].append(peak[1])
            else:
                self.ringds.append(peak[0])
                self.ringhkls[self.ringds[-1]]= [peak[1]]
        self.ringtol = tol

    def anglehkls(self,h1,h2):
        """
        Compute the angle between reciprocal lattice vectors h1, h2
        """
        g1 = np.dot(h1,np.dot(self.gi,h1))
        g2 = np.dot(h2,np.dot(self.gi,h2))
        g12= np.dot(h1,np.dot(self.gi,h2))
        costheta = g12/math.sqrt(g1*g2)
        try:
            return degrees(math.acos(costheta)),costheta
        except:
            if abs(costheta-1) < 1e-6:
                return 0.,1.0
            if abs(costheta+1) < 1e-6:
                return 180.,-1.0
            print("Error in unit cell class determining angle")
            print("h1",h1,"h2",h2,"Costheta=",costheta)
            raise

    def getanglehkls(self, ring1, ring2):
        """
        Cache the last pair called for
        """
        if self.anglehkl_rings == (ring1, ring2, self.ringtol):
            return self.anglehkl_cache
        else:
            self.anglehkl_rings = (ring1, ring2, self.ringtol)
            cache = []
            hcach = []
            for ha in self.ringhkls[self.ringds[ring1]]:
                for hb in self.ringhkls[self.ringds[ring2]]:
                    if ha == hb:
                        continue
                    hcach.append( ( ha, hb) )
                    # Note - returns angle, cosangle
                    cache.append( self.anglehkls(ha,hb)[1] )
            self.anglehkl_cache = [ hcach , np.array(cache) ]
            return self.anglehkl_cache


    def orient(self,ring1,g1,ring2,g2,verbose=0, all = True):
        """
        Compute an orientation matrix using cell parameters and the indexing
        of two reflections

        Orientation matrix:
        A matrix such that h = A^1 g
        define 3 vectors t1,t2,t3
        t1 is parallel to first peak (unit vector along g1)
        t2 is in the plane of both   (unit vector along g1x(g1xg2))
        t3 is perpendicular to both  (unit vector along g1xg2)
        """
        costheta = np.dot(g1,g2)/norm2(g2)/norm2(g1)
        if verbose==1: print("observed costheta",costheta)
        best=5.
        hab , angles_ab = self.getanglehkls( ring1, ring2 )
        diffs = abs( angles_ab - costheta )
        if all:
            order = np.argsort( diffs )
            best = [ order[0],]
            for i in range(1,len(order)):
                if diffs[order[i]] < diffs[order[0]]+1e-5:
                    best.append(order[i])
                else:
                    break
        else:
            best = [np.argmin( diffs ),]
        self.UBIlist = []
        UBlist=[]
        for b in best:
            try:
                h1, h2 = hab[b]
            except:
                print("hab=",hab)
                print("best=",b)
                print("len(hab)",len(hab))
                print("len(angles_ab)", len(angles_ab))
                print("costheta",costheta)
                print("abs( angles_ab - costheta ) )",abs( angles_ab - costheta )) 
                print("len(abs( angles_ab - costheta )) )",len(abs( angles_ab - costheta ) ))
                print(np.argmin( abs( angles_ab - costheta ) ))
                raise
            if verbose==1:
                print("Assigning h1",h1,g1,self.ds(h1),\
                    math.sqrt(np.dot(g1,g1)),\
                    self.ds(h1)-math.sqrt(np.dot(g1,g1)))
                print("Assigning h2",h2,g2,self.ds(h2),\
                    math.sqrt(np.dot(g2,g2)),\
                    self.ds(h1)-math.sqrt(np.dot(g1,g1)))
                print("Cos angle calc",self.anglehkls(h1,h2),"obs",costheta)
            try:
                h1c=np.dot(self.B,h1)    
                h2c=np.dot(self.B,h2)
                t1c=unit(h1c)
                t3c=unit(cross(h1c,h2c))
                t2c=unit(cross(h1c,t3c))
                t1g=unit(g1)
                t3g=unit(cross(g1,g2))
                t2g=unit(cross(g1,t3g))
                T_g = np.transpose(np.array([t1g,t2g,t3g]))  # Array are stored by rows and
                T_c = np.transpose(np.array([t1c,t2c,t3c]))  # these are columns
                U=np.dot(T_g , inv(T_c))
                UB=np.dot(U,self.B)
                UBI=inv(UB)
            except:
                logging.error("unitcell.orient h1 %s g1 %s h2 %s g2 %s self.B %s"%(
                     str(h1), str(g1), str(h2), str(g2), str(self.B)))
                print("angles_ab = ",angles_ab)
                print("hab",hab)
                print("best",b)
                print("ring1",ring1,"ring2",ring2)
                import traceback
                traceback.print_exc()
            if verbose==1:
                print("UBI")
                print(UBI)
                print("Grain gi")
                print(np.dot(np.transpose(UB),UB))
                print("Cell gi")
                print(self.gi)
                h=np.dot(UBI,g1)
                print("(%9.3f, %9.3f, %9.3f)"%(h[0],h[1],h[2]))
                h=np.dot(UBI,g2)
                print("(%9.3f, %9.3f, %9.3f)"%(h[0],h[1],h[2]))
            self.UBI=UBI
            self.UB=UB
            self.UBIlist.append(UBI)
            UBlist.append(UB)
        # trim to uniq list? What about small distortions...
        self.UBIlist = ubi_equiv( self.UBIlist, UBlist )

HKL0 = np.array( [ [0,0,1,1,-1,1,-1,0, 0, 1, -1, 1, 1],
                   [0,1,0,1, 1,0, 0,1,-1, 1,  1,-1, 1],
                   [1,0,0,0, 0,1, 1,1, 1, 1,  1, 1,-1] ], np.float ) # first unit cell
                 
def ubi_equiv( ubilist, ublist, tol=1e-8):
    """ Two ubi are considered equivalent if they both index the peaks
    in the HKL0 array exactly"""
    if len(ubilist) == 1:
        return ubilist
    order = np.argsort( [np.trace( ubi ) for ubi in ubilist ] ) # low to high
    uniq = [ ubilist[ order[-1] ], ]
    refgv = [ np.dot( ublist[ order[-1]] , HKL0 ), ]
    for i in order[:-1][::-1]:
        ubi = ubilist[i]
        score = 1
        for pks in refgv: # pks is (n,3) for an orientation
            hcalc = np.dot( ubi, pks )
            score = min(score, np.abs( np.round( hcalc ) - hcalc).sum() )
            if score <= tol: # matches a previous grain
                break
        if score > tol: # is a new orientation
            uniq.append( ubi )
            refgv.append( np.dot( np.linalg.inv( ubi ) , HKL0 ) )
    return uniq
    

def unitcell_from_parameters(pars):
    parnames = "_a _b _c alpha beta gamma".split()
    cell = unitcell([pars.get("cell_%s"%(s)) for s in parnames],
                     pars.get("cell_lattice_[P,A,B,C,I,F,R]"))
    return cell

if __name__=="__main__":
    import sys,time
    start=time.time()
    cell = unitcell([float(x) for x in sys.argv[1:7]],sys.argv[7])
    p=cell.gethkls(float(sys.argv[8]))
    n=len(p)
    print("Generated",n,"peaks in",time.time()-start,"/s")
    for k in p:
        try:
            print(k[1],k[0],1/k[0],k[1])
        except:
            pass



#!/usr/bin/env python
import sys, time
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from ImageD11.columnfile import columnfile 
 
class PeaksDisplay(QMainWindow):
    def __init__(self,parent=None):
        super(PeaksDisplay,self).__init__(parent)
        self.setWindowTitle('Peaks file')
        view = QTableView()
        tableData = PeaksTableModel()
        view.setModel(tableData)
        self.setCentralWidget(view)
 
 
class PeaksTableModel(QAbstractTableModel):
    """Model class that drives the population of tabular display"""
    def __init__(self):
        super(PeaksTableModel,self).__init__()
	start = time.time()
	self.colfile = columnfile( sys.argv[1] )
	print time.time()-start
 
    def rowCount(self,index=QModelIndex()):
        return self.colfile.nrows
 
    def readfile(self,fname):
        self.beginResetModel()
#	self.colfile = columnfile(fname)
        self.endResetModel()
 
    def columnCount(self,index=QModelIndex()):
        return self.colfile.ncols
 
    def data(self,index,role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
		return QVariant( float(self.colfile.bigarray[ index.column(), index.row()] ))
	else:
            return QVariant()
 
    def headerData(self,section,orientation,role=Qt.DisplayRole):
        if role != Qt.DisplayRole:
            return QVariant() 
        if orientation == Qt.Horizontal:
            return QVariant(self.colfile.titles[section])
        return QVariant(int(section + 1))
 
def start():
    app  = QApplication(sys.argv)
    appWin = PeaksDisplay()
    appWin.show()
    app.exec_()
 
if __name__ == "__main__":
    start()



from __future__ import print_function, division
from ImageD11 import cImageD11, indexing
try:
    from time import perf_counter_ns
except:
    from time import time
    def perf_counter_ns():
        return time()*1e9
import numpy as np
from PIL import ImageTk, Image
try:
    import tkinter as Tk
except:
    import Tkinter as Tk
    
i = indexing.indexer()
i.readgvfile("eu3.gve")
gve = i.gv.copy().astype(np.float)

w,h = 512,512
rgba = np.zeros( (w, h, 4), np.uint8 )
u = np.eye(3, dtype=np.float).ravel()
t0 = perf_counter_ns()
cImageD11.splat( rgba, gve, u, 2 )
t1 = perf_counter_ns()
im = Image.fromarray( rgba, mode="RGBA")
r = Tk.Tk()
myimg = ImageTk.PhotoImage( im )
l = Tk.Label( master=r, image = myimg )
l.pack()

s = np.sin(np.radians(2))
c = np.cos(np.radians(2))
ry = np.array( [[c,0,s],[0,1,0],[-s,0,c]] )
s = np.sin(np.radians(2/np.sqrt(2)))
c = np.cos(np.radians(2/np.sqrt(2)))
rx = np.array( [[1,0,0],[0,c,s],[0,-s,c]] )
rz = np.dot( ry, rx)
def rotate():
    global u, rz, rgba, gve, myimg, l
    t0 = perf_counter_ns()
    u = np.dot( rz, np.reshape(u,(3,3) )).ravel()
    cImageD11.splat( rgba, gve, u, 2 )
    im = Image.fromarray( rgba, mode="RGBA") 
    myimg.paste(im)
    r.after( 60, rotate )
    t1 = perf_counter_ns()
    print("%8.3f ms per frame"%((t1-t0)/1e6),end="\r")

r.after(60, rotate)
r.mainloop()





import sys, os
import numpy as np, fabio
import silx.gui.qt , silx.gui.plot

from ImageD11.peakmerge import peakmerger
qapp=None

class sptview(object):

    def __init__(self, fname=None):
        self.pm = peakmerger()
        self.fname = fname
        self.pm.readpeaks( fname )
        self.select_image(0)
        self.drawUI()

    def select_image(self, i):
        if i < 0 or  i > len(self.pm.images)-1:
            return False
        im = self.pm.images[i]
        self.currentnum = i
        j = im.imagenumber
        self.pm.harvestpeaks(  numlim=(j-.1,j+0.1) )
        self.frame = fabio.open(im.name)       
        self.pkx = [ p.x for p in self.pm.allpeaks ]
        self.pky = [ p.y for p in self.pm.allpeaks ]
        self.pkI = [ p.avg for p in self.pm.allpeaks ]
        return True

    def next(self):
        if self.select_image( self.currentnum + 1):
            self.changeframe()
        else:
            self.warn("No next image")

    def prev(self):
        if self.select_image( self.currentnum - 1):
            self.changeframe()
        else:
            self.warn("No previous image")

    def drawUI(self):
        self.widget = silx.gui.qt.QWidget()
        self.widget.setWindowTitle("spot viewer")
        self.layout = silx.gui.qt.QGridLayout()
        self.widget.setLayout( self.layout )
        self.spot_label = silx.gui.qt.QLabel( self.fname )
        self.framelabel = silx.gui.qt.QLabel( self.frame.filename )
        self.framelabel.setAlignment( silx.gui.qt.Qt.AlignCenter )
        self.layout.addWidget(self.spot_label, 0, 0, 1, 3)
        self.layout.addWidget(self.framelabel, 1, 1)
        self.n = silx.gui.qt.QPushButton( "next" )
        self.p = silx.gui.qt.QPushButton( "prev" )
        self.n.clicked.connect( self.next )
        self.p.clicked.connect( self.prev )
        self.layout.addWidget(self.p, 1, 0 )
        self.layout.addWidget(self.n, 1, 2 )
        self.plot = silx.gui.plot.Plot2D(  backend='gl')
        self.imglabel = self.plot.addImage( self.frame.data, z=0, origin=(-0.5,-0.5),
            colormap=silx.gui.colors.Colormap(name='magma'))
        self.plotlabel = self.plot.addScatter( self.pky, self.pkx, self.pkI, z=1, symbol='+',
            colormap=silx.gui.colors.Colormap(name='viridis'))
        self.plot.getScatter( self.plotlabel ).setSymbolSize(10)
        self.plot.setKeepDataAspectRatio(True)
        self.layout.addWidget(self.plot, 2, 0, 1, 3)
        self.widget.show()
        self.plot.show()

    def changeframe(self):
        d = self.frame.data
        im = self.plot.getImage( self.imglabel )
        im.setData(d)
        pl = self.plot.getScatter( self.plotlabel )
        pl.setData( self.pky, self.pkx, self.pkI )
        self.framelabel.setText( self.frame.filename )

    def warn(self, message):
        silx.gui.qt.QMessageBox.warning(None, "Warning", message)

if __name__=="__main__":
    qapp = silx.gui.qt.QApplication([])
    s=sptview( sys.argv[1] )
    s.widget.show()
    sys.exit(qapp.exec_())



from ImageD11.columnfile import columnfile
c = columnfile("mainphase.flt")
c.addcolumn(c.omega.copy(), "image_number")
c.omega[:] = (c.image_number/147).astype(int)
c.addcolumn(0.2*(c.image_number - 147*c.omega),"xpos")
plot(c.omega,c.sum_intensity,",")
c.writefile("mainphase_fixed_omega.flt")




from __future__ import print_function


"""
Routines for finding the lattice symmetry from a ubi file
"""
import numpy as np



def cosangle_rr_vec( ubi, v1, v2 ):
    """
    Returns the cosine of the angle between the vector, v, in
    real and reciprocal space.

    In monoclinic-b symmetry the real space b and reciprocal space
    b* axes are parallel.
    This generalises, so all 2-fold lattice symmetries are parallel
    See:
    J. Appl. Cryst. (1982). 15, 255-259    [ doi:10.1107/S0021889882011959 ]
    "The derivation of the axes of the conventional unit cell from the
    dimensions of the Buerger-reduced cell"
    """
    real = np.dot( ubi.T, v1 )
    reci = np.dot( np.linalg.inv(ubi) , v2 )
    return np.dot( real, reci )/np.sqrt(
        np.dot(real, real) * np.dot(reci, reci) )

# All the possible indices of lattice rows and planes
rows_planes = [ (h, k, l) for h in range(-2,3)
                          for k in range(-2,3)
                          for l in range(-2,3) ]
rows_planes.remove( (0,0,0) )
rows_planes = np.array( rows_planes )

from ImageD11.rc_array import rc_array

def find_2_folds( ubi , tol = 1.0, debug = False):
    """
    Scans over different directions to find all the potential
    2-fold axes.
    Default tolerance is 0.5 degrees.
    """
    tol_c = abs(1.0 - np.cos( tol * np.pi / 180.0))
    ub = np.linalg.inv( ubi )

    real_vecs = np.dot( ubi, rows_planes.T ).T
    reci_vecs = np.dot( ub.T , rows_planes.T ).T
    
    real_lens = np.sqrt((real_vecs*real_vecs).sum(axis = 1))
    reci_lens = np.sqrt((reci_vecs*reci_vecs).sum(axis = 1))
    nvec = len(rows_planes)
    pairs = []
    for v1, l1, i in zip(real_vecs, real_lens, list(range(nvec))):
        print(v1,l1,i)
        for v2, l2, j in zip(reci_vecs, reci_lens, list(range(nvec))):
            #if j < i:
            #    continue
            c = np.dot(v1, v2)/ l1/ l2
            if abs( 1 - c ) < tol_c:
                print(i, j, c, rows_planes[i],rows_planes[j])
                pairs.append( (i,j) )
    print(pairs)
    return pairs

    


from ImageD11.lattice_reduction import mod as lattice_mod


def r2sweep( vl ):
    """
    A sweep of subtracting vectors from each other
    """
    vn = np.asarray( vl ).copy()
    nv = len(vl)
    for i in range(nv):
        for j in range(i+1, i+nv):
            k = j%nv
            assert i != k
            vn[k] = lattice_mod( vn[k], vn[i] )
    return vn


def reduce_two_folds( vecs ):
    """
    Try to make a uniq list of 2-fold axes (eg, recognise
    2x and -1x etc)
    """
    vn = r2sweep( vecs )
    uniq = []
    for v in vn:
        if np.dot( v, v) > 0:
            if v.sum( ) > 0:
                uniq.append( v )
            else:
                uniq.append( -v )
    return uniq

import xfab.tools


mono = ( 1, 2, 3, 89.8, 107, 90.02 )
ortho = ( 1.5, 2, 3, 90., 90., 90. )
def transform( cell, matrix):
    return xfab.tools.ubi_to_cell(
        np.dot(  np.linalg.inv(matrix), 
                 xfab.tools.form_a_mat( cell )))

testcases = {
    
    'monoclinic'   : [ mono , (0,1,0) ],
    'monoclinic_c'   : [ transform( mono, [[1,1,0],[-1,1,0],[0,0,1]]),
                         (1,-1,0) ],
    
    'orthorhombic' : [ ortho ,
                       (1,0,0),  (0,1,0),  (0,0,1) ],
    
    'orthorhombic_c' : [transform( ortho, [[1,1,0],[-1,1,0],[0,0,1]]) ,
                        (1,1,0),  (1,-1,0),  (0,0,1) ],
    
    'orthorhombic_i' : [ transform( ortho, [[0,1,1],[1,0,1],[1,1,0]]),
                          (1,1,-1),  (1,-1,1),  (-1,1,1) ],
    
    'orthorhombic_f' : [ transform( ortho, [[-1,1,1],[1,-1,1],[1,1,-1]] ),
                         (1,0,1),  (0,1,1),  (1,1,0) ],
    
    
    }


def test_reduce( name, cell , expected ):
    print(("%6.3f  "*6)%tuple(cell), end=' ')
    ubi = xfab.tools.form_a_mat( cell ).T
    twofolds = find_2_folds( ubi )
    lines  = [ rows_planes[t[0]] for t in twofolds]
    planes = [ rows_planes[t[1]] for t in twofolds]
    uniql = reduce_two_folds( planes )
    uniqp = reduce_two_folds( lines ) 
    # print name, uniq
    if len(expected) != len( uniql ):
        print(name + " wrong number of axes ")
        print(uniql)
        print(uniqp)
    for ul, up in zip(uniql, uniqp):
        ok = (tuple(ul) in expected) or (tuple(-ul) in expected)
        if not ok:
            print("PROBLEM",name, ul, up, expected)
            
def runtests():
    np.seterr(all='raise')
    names = list(testcases.keys())
    names.sort()
    for name in names:
        cell = testcases[name][0]
        expected = testcases[name][1:]

        print("Testing",name, end=' ')
        test_reduce( name, cell, expected)
        print("passed")
        

    
if __name__=="__main__":
    runtests()





import pydoc, os, ImageD11,inspect, shutil
pydoc.writedoc("ImageD11")
shutil.copyfile("ImageD11.html","index.html")
done=[None]
for file in os.listdir(ImageD11.__path__[0]):
    path = os.path.join(ImageD11.__path__[0], file)
    modname = inspect.getmodulename(file)
    if modname not in done: # and modname != '__init__':
        pydoc.writedoc("ImageD11."+modname)
        done.append(modname)



from __future__ import print_function, division

from ImageD11 import sym_u
from ImageD11 import grain
import sys, numpy as np, xfab.tools

print ("# Usage: %s grains1.map grains2.map group angletol  distancetol"%(sys.argv[0]))
print ("# Matches grains using symmetry group from:  ")
print ("#  [cubic|hexagonal|trigonal|rhombohedralP|tetragonal|orthorhombic|monoclinic_[a|b|c]|triclinic]")


g1l = grain.read_grain_file(sys.argv[1])
g2l = grain.read_grain_file(sys.argv[2])
for g in g1l + g2l:
    if g.translation is None:
        g.translation = np.zeros(3)

print( "read in the grains")

try:
    h = getattr( sym_u, sys.argv[3] )()
except:
    print( "# No group!")
    print( "#Using cubic")
    h = sym_u.cubic()
    print ("Made a cubic group")

print ( "# Symmetry considered")
for o in h.group:
    print(o)

try:
    tolangle = float(sys.argv[4])
except:
    tolangle = 1.0

try:
    toldist = float(sys.argv[5])
except:
    toldist = 100.0

dt0 = np.array([0.,0.,0.])
if len(sys.argv)>=9:
    try:
        dt0 = np.array([float(x) for x in sys.argv[6:9]])
    except:
        raise



da=np.zeros( (len(g1l),len(g2l)), np.float)
dt2=np.zeros( (len(g1l),len(g2l)), np.float)

for g in g1l + g2l:
    g.u = xfab.tools.ubi_to_u_b(g.ubi)[0]
    assert (abs(np.dot(g.u, g.u.T) - np.eye(3)).ravel()).sum() < 1e-6
    
dtsum = np.zeros(3)
ndt = 0
toldist2 = toldist*toldist

g2t = np.array( [g2.translation for g2 in g2l] )

for i,g1 in enumerate(g1l):
    minangle = 180.0
    best = None
    symubis = [np.dot(o, g1.ubi) for o in h.group]
#    symus   = [xfab.tools.ubi_to_u_b(ubi)[0] for ubi in symubis]
    asymusT   = np.array([xfab.tools.ubi_to_u_b(ubi)[0].T for ubi in symubis])
    printed = False
    trace = np.trace
    pi = np.pi

    dT = g1.translation - g2t
    dT2 = (dT*dT).sum(axis=1)

    inds = np.arange( len(g2t) )[dT2 < toldist2]

    for j in inds:
        g2 = g2l[j]
        
        dt = g1.translation - g2.translation 
        #dt2 = (dt*dt).sum()
        #if dt2 > toldist2:
        #    continue
        
        sg = None
        aumis = np.dot(asymusT, g2.u)
        arg = (aumis[:,0,0]+aumis[:,1,1]+aumis[:,2,2] - 1. )/2.
        asymangle = np.arccos(np.clip(arg, -1, 1))
        sg = np.argmin(asymangle)
        angle = asymangle[sg]*180.0/pi        
        # translation
        if angle < tolangle :
            #print "# Found a match!", h.group[sg].ravel()
            dtsum += dt
            ndt += 1
            print( i,j,"angle  %.4f"%(angle),"distance %.3f"%(np.sqrt(dT2[j])),"\t", end="")
            print( 3*"%.1f "%tuple(dt),"\t",end="")
            print( 3*"%.1f "%tuple(dtsum/ndt))
            printed = True
    if not printed:
        print ("# not matched %d "%(i))







from __future__ import print_function

# ImageD11_v1.0 Software for beamline ID11
# Copyright (C) 2005-2018  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
Setup script

Do:
  python setup.py build --force
   -> it will build static libraries in the src folder
   -> compile wrappers them here
"""

# For pip / bdist_wheel etc
import setuptools
import os, sys
# Need to use numpy.distutils so it works for f2py
from numpy.distutils.core import setup, Extension
from numpy import get_include
from distutils.command.sdist import sdist


# Get the path for the static libraries        
import src.bldlib, src.create_cpu_check

def get_version():
    with open("ImageD11src/__init__.py","r") as f:
        for line in f.readlines():
            if line.find("__version__")>-1:
                return eval(line.split("=")[1].strip())

print("Building version |%s|"%get_version())

def build_clibs():
    """ 
    Should learn to use build_clib at some point
    Have not figured out to have per c-file args without
    caching the o-files (avx vs sse issue)
    """
    if not src.bldlib.need_build:
        return
    os.chdir("src")
    print("Call create_cpu_check")
    src.create_cpu_check.main()
    # os.system(sys.executable+" write_check.py")
    print("Call bldlib")
    # transmit compiler from command line
    # ok = os.system(sys.executable+" bldlib.py "+ " ".join(sys.argv))
    ok = src.bldlib.main()
    os.chdir("..")
    if ok != 0:
        print("Return was",ok)
        sys.exit(ok)
    else:
        print("Seems to build OK")
    src.bldlib.need_build = False

# Issue 66, try to get pip install to work.
#   ... but only recompile once and when needed
for arg in ("build", "bdist_wheel", "bdist_egg", "develop","--force"):
    if arg in sys.argv and src.bldlib.need_build:
        build_clibs()


ekwds = { 'include_dirs' : [get_include(), 'src' ],
          'library_dirs' : ['./src'],
          'extra_compile_args' : src.bldlib.arg + \
               ["-DF2PY_REPORT_ON_ARRAY_COPY=1.",],
          'extra_link_args' : src.bldlib.arg + \
               ["-DF2PY_REPORT_ON_ARRAY_COPY=1.",],
        }

# Compiled extensions:
extensions = [ Extension( "cImageD11_sse2", 
                          sources = ["src/cImageD11_sse2.pyf",],
                          libraries = [src.bldlib.sse2libname],
                          **ekwds ),
               Extension( "cImageD11_avx",
                          sources = ["src/cImageD11_avx.pyf",],
                          libraries = [src.bldlib.avx2libname],
                          **ekwds) ]




# Things we depend on. This generally borks things if pip
# tries to do source installs of all of these.

needed =[
    "six",
    "numpy",
    "scipy",
    "pillow",
    "h5py",
    "matplotlib",
    "xfab>=0.0.4",
    "fabio",
    "pyopengl",
    "pyopengltk",
    ]
    # , "FitAllB", ... ]

# See the distutils docs...
setup(name='ImageD11',
      version=get_version(),
      author='Jon Wright',
      author_email='wright@esrf.fr',
      cmdclass={'sdist': sdist},
      description='ImageD11',
      license = "GPL",
      ext_package = "ImageD11",   # Puts extensions in the ImageD11 directory
      ext_modules = extensions,
      install_requires = needed,
      packages = ["ImageD11", "ImageD11.tkGui", "ImageD11.silxGui"],
      package_dir = {"ImageD11":"ImageD11src"},
      url = "http://github.com/jonwright/ImageD11",
      package_data = {"ImageD11" : ["doc/*.html", "data/*" ]},
      scripts = ["ImageD11src/rsv_mapper.py",
                 "ImageD11src/tkGui/plot3d.py",
                 "scripts/peaksearch.py",
                 "scripts/fitgrain.py",
                 "scripts/tomapper.py",
                 "scripts/ubi2cellpars.py",
                 "scripts/filtergrain.py",
                 "scripts/pars_2_sweeper.py",
                 "scripts/ImageD11_2_shelx.py",
                 "scripts/fit2dcake.py",
                 "scripts/fix_spline.py",
                 "scripts/edfheader.py",
                 "scripts/huber2bruker.py",
                 "scripts/id11_summarize.py",
                 "scripts/ImageD11_gui.py",
                 "scripts/bgmaker.py",
                 "scripts/merge_flt.py",
                 "scripts/makemap.py",
                 "scripts/plotlayer.py",
                 "scripts/plotedf.py",
                 "scripts/plotgrainhist.py",
                 "scripts/rubber.py",
                 "scripts/plotImageD11map.py",
                 "scripts/cutgrains.py",
                 "scripts/index_unknown.py",
                 "scripts/spatialfix.py",
       	         "scripts/refine_em.py",
                 "scripts/avg_par.py",
                 "scripts/powderimagetopeaks.py"])







from __future__ import print_function

# ImageD11_v1.0 Software for beamline ID11
# Copyright (C) 2005-2007  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0211-1307  USA

try:
    import Queue
except:
    # python 3?
    import queue as Queue 
    
import threading
# global stop_now
stop_now = False

class ImageD11_thread(threading.Thread):
    """ Add a stopping mechanism for unhandled exceptions """
    def __init__(self, myname="ImageD11_thread"):
        self.myname=myname
        threading.Thread.__init__(self)
    def run(self):
        global stop_now
        try:
            self.ImageD11_run()
        except:
            stop_now = True
            raise
    def ImageD11_stop_now(self):
        global stop_now
        if stop_now:
            print( "Got a stop in",self.myname)
        return stop_now
